"""definition of each subcommands"""

from inspect import getdoc
from itertools import zip_longest
from math import ceil
from shutil import get_terminal_size
from textwrap import TextWrapper
from . import constants, misc, field, rprof, time_series, plates, stagyydata
from . import __version__


def field_cmd(args):
    """Plot scalar and vector fields"""
    misc.plot_backend(args)
    field.field_cmd(args)


def rprof_cmd(args):
    """Plot radial profiles"""
    misc.plot_backend(args)
    rprof.rprof_cmd(args)


def time_cmd(args):
    """Plot time series"""
    misc.plot_backend(args)
    time_series.time_cmd(args)


def plates_cmd(args):
    """Plate analysis"""
    misc.plot_backend(args)
    if args.plot is not None:
        for var, meta in constants.PLATES_VAR_LIST.items():
            misc.set_arg(args, meta.arg, var in args.plot)
    plates.plates_cmd(args)


def info_cmd(args):
    """Print basic information about StagYY run"""
    sdat = stagyydata.StagyyData(args.path)
    lsnap = sdat.snaps.last
    lstep = sdat.steps.last
    lfields = []
    for fvar in constants.FIELD_VARS:
        if lsnap.fields[fvar] is not None:
            lfields.append(fvar)
    print('StagYY run in {}'.format(sdat.path))
    print('Last timestep:',
          '  istep: {}'.format(lstep.istep),
          '  time:  {}'.format(lstep.timeinfo['t']),
          '  <T>:   {}'.format(lstep.timeinfo['Tmean']),
          sep='\n')
    print('Last snapshot (istep {}):'.format(lsnap.istep),
          '  isnap: {}'.format(lsnap.isnap),
          '  time:  {}'.format(lsnap.timeinfo['t']),
          '  output fields: {}'.format(','.join(lfields)),
          sep='\n')


def _layout(dict_vars, dict_vars_extra):
    """Print nicely [(var, description)] from *_VARS and *_VARS__EXTRA"""
    desc = [(v, m.description) for v, m in dict_vars.items()]
    desc.extend((v, getdoc(m.description)) for v, m in dict_vars_extra.items())
    termw = get_terminal_size().columns
    ncols = (termw + 1) // 27  # min width of 26
    colw = (termw + 1) // ncols - 1
    ncols = min(ncols, len(desc))

    wrapper = TextWrapper(width=colw)
    lines = []
    for varname, description in desc:
        wrapper.subsequent_indent = ' ' * (len(varname) + 2)
        lines.extend(wrapper.wrap('{}: {}'.format(varname, description)))

    chunks = []
    for rem_col in range(ncols, 1, -1):
        isep = ceil(len(lines) / rem_col)
        while isep < len(lines) and lines[isep][0] == ' ':
            isep += 1
        chunks.append(lines[:isep])
        lines = lines[isep:]
    chunks.append(lines)
    lines = zip_longest(*chunks, fillvalue='')

    fmt = '|'.join(['{{:{}}}'.format(colw)] * (ncols - 1))
    fmt += '|{}'
    print(*(fmt.format(*line) for line in lines), sep='\n')


def var_cmd(_):
    """Print a list of available variables"""
    print('field:')
    _layout(constants.FIELD_VARS, constants.FIELD_VARS_EXTRA)
    print()
    print('rprof:')
    _layout(constants.RPROF_VARS, constants.RPROF_VARS_EXTRA)
    print()
    print('time:')
    _layout(constants.TIME_VARS, constants.TIME_VARS_EXTRA)
    print()
    print('plates:')
    _layout(constants.PLATES_VAR_LIST, {})


def version_cmd(_):
    """Print StagPy version"""
    print('stagpy version: {}'.format(__version__))



"""Parsers of StagYY output files"""
from functools import partial
from itertools import product, repeat
from operator import itemgetter
import re
import struct
import numpy as np
import pandas as pd


def time_series(timefile, colnames):
    """Read temporal series from time.dat"""
    if not timefile.is_file():
        return None
    data = np.genfromtxt(timefile, skip_header=1)

    # detect useless lines produced when run is restarted
    rows_to_del = []
    for irow in range(1, len(data)):
        iprev = irow - 1
        while int(data[irow, 0]) <= int(data[iprev, 0]):
            rows_to_del.append(iprev)
            iprev -= 1
    data = np.delete(data, rows_to_del, 0)
    ncols = data.shape[1] - 1
    colnames = colnames[:ncols] + list(range(0, ncols - len(colnames)))

    isteps = map(int, data[:, 0])
    return pd.DataFrame(data[:, 1:], index=isteps, columns=colnames)


def _extract_rsnap_isteps(rproffile):
    """Extract istep and compute list of rows to delete"""
    step_regex = re.compile(r'^\*+step:\s*(\d+) ; time =\s*(\S+)')
    isteps = []  # list of (istep, time, nz)
    rows_to_del = []
    line = ' '
    with rproffile.open() as stream:
        while line[0] != '*':
            line = stream.readline()
        match = step_regex.match(line)
        istep = int(match.group(1))
        time = float(match.group(2))
        nlines = 0
        iline = 0
        for line in stream:
            if line[0] == '*':
                isteps.append((istep, time, nlines))
                match = step_regex.match(line)
                istep = int(match.group(1))
                time = float(match.group(2))
                nlines = 0
                # remove useless lines produced when run is restarted
                nrows_to_del = 0
                while isteps and istep <= isteps[-1][0]:
                    nrows_to_del += isteps.pop()[-1]
                rows_to_del.extend(range(iline - nrows_to_del, iline))
            else:
                nlines += 1
                iline += 1
        isteps.append((istep, time, nlines))
    return isteps, rows_to_del


def rprof(rproffile, colnames):
    """Extract radial profiles data"""
    if not rproffile.is_file():
        return None, None
    data = np.genfromtxt(rproffile, comments='*')

    isteps, rows_to_del = _extract_rsnap_isteps(rproffile)
    data = np.delete(data, rows_to_del, 0)

    ncols = data.shape[1]
    colnames = colnames[:ncols] + list(range(0, ncols - len(colnames)))

    id_arr = [[], []]
    for istep, _, n_z in isteps:
        id_arr[0].extend(repeat(istep, n_z))
        id_arr[1].extend(range(n_z))

    df_rprof = pd.DataFrame(data, index=id_arr, columns=colnames)
    df_times = pd.DataFrame(list(map(itemgetter(1), isteps)),
                            index=map(itemgetter(0), isteps))
    return df_rprof, df_times


def _readbin(fid, fmt='i', nwords=1, file64=False):
    """Read n words of 4 or 8 bytes with fmt format.

    fmt: 'i' or 'f' (integer or float)
    4 or 8 bytes: depends on header

    Return an array of elements if more than one element.

    Default: read 1 word formatted as an integer.
    """
    if file64:
        nbytes = 8
        fmt = fmt.replace('i', 'q')
        fmt = fmt.replace('f', 'd')
    else:
        nbytes = 4
    elts = np.array(struct.unpack(fmt * nwords, fid.read(nwords * nbytes)))
    if len(elts) == 1:
        elts = elts[0]
    return elts


def fields(fieldfile, only_header=False, only_istep=False):
    """Extract fields data"""
    # something to skip header?
    if not fieldfile.is_file():
        return None
    header = {}
    with fieldfile.open('rb') as fid:
        readbin = partial(_readbin, fid)
        magic = readbin()
        if magic > 8000:  # 64 bits
            magic -= 8000
            readbin()  # need to read 4 more bytes
            readbin = partial(readbin, file64=True)

        # check nb components
        nval = 1
        if magic > 400:
            nval = 4
        elif magic > 300:
            nval = 3

        magic %= 100

        # extra ghost point in horizontal direction
        header['xyp'] = int(magic >= 9 and nval == 4)

        # total number of values in relevant space basis
        # (e1, e2, e3) = (theta, phi, radius) in spherical geometry
        #              = (x, y, z)            in cartesian geometry
        header['nts'] = readbin(nwords=3)

        # number of blocks, 2 for yinyang or cubed sphere
        header['ntb'] = readbin() if magic >= 7 else 1

        # aspect ratio
        header['aspect'] = readbin('f', 2)

        # number of parallel subdomains
        header['ncs'] = readbin(nwords=3)  # (e1, e2, e3) space
        header['ncb'] = readbin() if magic >= 8 else 1  # blocks

        # r - coordinates
        # rgeom[0:self.nrtot+1, 0] are edge radial position
        # rgeom[0:self.nrtot, 1] are cell-center radial position
        if magic >= 2:
            header['rgeom'] = readbin('f', header['nts'][2] * 2 + 1)
        else:
            header['rgeom'] = np.array(range(0, header['nts'][2] * 2 + 1))\
                * 0.5 / header['nts'][2]
        header['rgeom'].resize((header['nts'][2] + 1, 2))

        header['rcmb'] = readbin('f') if magic >= 7 else None

        header['ti_step'] = readbin() if magic >= 3 else 0
        if only_istep:
            return header['ti_step']
        header['ti_ad'] = readbin('f') if magic >= 3 else 0
        header['erupta_total'] = readbin('f') if magic >= 5 else 0
        header['bot_temp'] = readbin('f') if magic >= 6 else 1

        if magic >= 4:
            header['e1_coord'] = readbin('f', header['nts'][0])
            header['e2_coord'] = readbin('f', header['nts'][1])
            header['e3_coord'] = readbin('f', header['nts'][2])
        else:
            # could construct them from other info
            raise ValueError('magic >= 4 expected to get grid geometry')

        if only_header:
            return header

        # READ FIELDS
        # number of points in (e1, e2, e3) directions PER CPU
        npc = header['nts'] // header['ncs']
        # number of blocks per cpu
        nbk = header['ntb'] // header['ncb']
        # number of values per 'read' block
        npi = (npc[0] + header['xyp']) * (npc[1] + header['xyp']) * npc[2] * \
            nbk * nval

        header['scalefac'] = readbin('f') if nval > 1 else 1

        flds = np.zeros((nval,
                         header['nts'][0] + header['xyp'],
                         header['nts'][1] + header['xyp'],
                         header['nts'][2],
                         header['ntb']))

        # loop over parallel subdomains
        for icpu in product(range(header['ncb']),
                            range(header['ncs'][2]),
                            range(header['ncs'][1]),
                            range(header['ncs'][0])):
            # read the data for one CPU
            data_cpu = readbin('f', npi) * header['scalefac']

            # icpu is (icpu block, icpu z, icpu y, icpu x)
            # data from file is transposed to obtained a field
            # array indexed with (x, y, z, block), as in StagYY
            flds[:,
                 icpu[3] * npc[0]:(icpu[3] + 1) * npc[0] + header['xyp'],  # x
                 icpu[2] * npc[1]:(icpu[2] + 1) * npc[1] + header['xyp'],  # y
                 icpu[1] * npc[2]:(icpu[1] + 1) * npc[2],  # z
                 icpu[0] * nbk:(icpu[0] + 1) * nbk  # block
                 ] = np.transpose(data_cpu.reshape(
                     (nbk, npc[2], npc[1] + header['xyp'],
                      npc[0] + header['xyp'], nval)))
    return header, flds



# PYTHON_ARGCOMPLETE_OK
"""The stagpy module is callable"""

import importlib
import signal
import sys


def sigint_handler(*_):
    """SIGINT handler"""
    print('\nSo long, and thanks for all the fish.')
    sys.exit()


def main():
    """StagPy entry point"""
    signal.signal(signal.SIGINT, sigint_handler)
    config = importlib.import_module('stagpy.config')
    args = config.parse_args()
    args.func(args)

if __name__ == '__main__':
    main()



"""Plots position of subduction and ridge at the surface.

Date: 2016/26/01
"""
from copy import deepcopy
import sys
import os.path
import numpy as np
from scipy.signal import argrelextrema
from . import constants, field, misc
from .stagyydata import StagyyData


def detect_plates_vzcheck(step, seuil_memz):
    """detect plates and check with vz and plate size"""
    v_z = step.fields['v3'][0, :, :, 0]
    v_x = step.fields['v2'][0, :, :, 0]
    h2o = step.fields['wtr'][0, :, :, 0]
    tcell = step.fields['T'][0, :, :, 0]
    n_z = step.geom.nztot
    nphi = step.geom.nptot  # -1? should be OK, ghost not included
    rcmb = max(0, step.geom.rcmb)
    radius = step.geom.r_coord + rcmb
    radiusgrid = step.geom.rgeom[:, 0] + rcmb
    dphi = 1 / nphi

    # water profile
    water_profile = np.mean(h2o, axis=0)

    # calculing tmean
    tmean = step.timeinfo.loc['Tmean']

    # calculing temperature on the grid and vz_mean/v_rms
    v_rms = step.timeinfo.loc['vrms']
    vz_mean = 0
    tgrid = np.zeros((nphi, n_z + 1))
    tgrid[:, 0] = 1
    for i_z in range(1, n_z):
        for phi in range(nphi):
            tgrid[phi, i_z] = (
                tcell[phi, i_z - 1] *
                (radiusgrid[i_z] - radius[i_z - 1]) + tcell[phi, i_z] *
                (-radiusgrid[i_z] + radius[i_z])) / (radius[i_z] -
                                                     radius[i_z - 1])
            vz_mean += abs(v_z[phi, i_z]) / (nphi * n_z)
    print(v_rms, vz_mean)

    flux_c = n_z * [0]
    for i_z in range(1, n_z - 1):
        for phi in range(nphi):
            flux_c[i_z] += (tgrid[phi, i_z] - tmean) * \
                v_z[phi, i_z] * radiusgrid[i_z] * dphi

    # checking stagnant lid
    stagnant_lid = True
    max_flx = np.max(flux_c)
    for i_z in range(n_z - n_z // 20, n_z):
        if abs(flux_c[i_z]) > max_flx / 50:
            stagnant_lid = False
            break
    if stagnant_lid:
        print('stagnant lid')
        sys.exit()
    else:
        # verifying horizontal plate speed and closeness of plates
        dvphi = nphi * [0]
        dvx_thres = 16 * v_rms

        for phi in range(0, nphi):
            dvphi[phi] = (v_x[phi, n_z - 1] -
                          v_x[phi - 1, n_z - 1]) / ((1 + rcmb) * dphi)
        limits = []
        for phi in range(0, nphi - nphi // 33):
            mark = True
            for i in range(phi - nphi // 33, phi + nphi // 33):
                if abs(dvphi[i]) > abs(dvphi[phi]):
                    mark = False
            if mark and abs(dvphi[phi]) >= dvx_thres:
                limits.append(phi)
        for phi in range(nphi - nphi // 33 + 1, nphi):
            mark = True
            for i in range(phi - nphi // 33 - nphi, phi + nphi // 33 - nphi):
                if abs(dvphi[i]) > abs(dvphi[phi]):
                    mark = False
            if mark and abs(dvphi[phi]) >= dvx_thres:
                limits.append(phi)
        print(limits)

        # verifying vertical speed
        k = 0
        for i in range(len(limits)):
            vzm = 0
            phi = limits[i - k]
            if phi == nphi - 1:
                for i_z in range(1, n_z):
                    vzm += (abs(v_z[phi, i_z]) +
                            abs(v_z[phi - 1, i_z]) +
                            abs(v_z[0, i_z])) / (n_z * 3)
            else:
                for i_z in range(0, n_z):
                    vzm += (abs(v_z[phi, i_z]) +
                            abs(v_z[phi - 1, i_z]) +
                            abs(v_z[phi + 1, i_z])) / (n_z * 3)

            if seuil_memz != 0:
                vz_thres = vz_mean * 0.1 + seuil_memz / 2
            else:
                vz_thres = vz_mean * 0
            if vzm < vz_thres:
                limits.remove(phi)
                k += 1
        print(limits)

        print('\n')
    return limits, nphi, dvphi, vz_thres, v_x[:, n_z - 1], water_profile


def detect_plates(args, step, vrms_surface, fids, time):
    """detect plates using derivative of horizontal velocity"""
    vphi = step.fields['v2'][0, :, :, 0]
    ph_coord = step.geom.p_coord

    if step.sdat.par['boundaries']['air_layer']:
        dsa = step.sdat.par['boundaries']['air_thickness']
        # we are a bit below the surface; should check if you are in the
        # thermal boundary layer
        indsurf = np.argmin(abs((1 - dsa) - step.geom.r_coord)) - 4
    else:
        dsa = 0.
        indsurf = -1

    vph2 = 0.5 * (vphi + np.roll(vphi, 1, 0))  # interpolate to the same phi
    # velocity derivation
    dvph2 = (np.diff(vph2[:, indsurf]) / (ph_coord[0] * 2.))

    io_surface(step.isnap, time, fids[6], dvph2)
    io_surface(step.isnap, time, fids[7], vph2[:-1, indsurf])

    # prepare stuff to find trenches and ridges
    myorder_trench = 15 if step.sdat.par['boundaries']['air_layer'] else 10
    myorder_ridge = 20  # threshold

    # finding trenches
    pom2 = deepcopy(dvph2)
    if step.sdat.par['boundaries']['air_layer']:
        maskbigdvel = -30 * vrms_surface
    else:
        maskbigdvel = -10 * vrms_surface
    pom2[pom2 > maskbigdvel] = maskbigdvel
    argless_dv = argrelextrema(
        pom2, np.less, order=myorder_trench, mode='wrap')[0]
    trench = ph_coord[argless_dv]
    velocity_trench = vph2[argless_dv, indsurf]
    dv_trench = dvph2[argless_dv]

    # finding ridges
    pom2 = deepcopy(dvph2)
    masksmalldvel = np.amax(dvph2) * 0.2
    pom2[pom2 < masksmalldvel] = masksmalldvel
    arggreat_dv = argrelextrema(
        pom2, np.greater, order=myorder_ridge, mode='wrap')[0]
    ridge = ph_coord[arggreat_dv]

    # elimination of ridges that are too close to trench
    argdel = []
    if len(trench) and len(ridge):
        for i in range(len(ridge)):
            mdistance = np.amin(abs(trench - ridge[i]))
            if mdistance < 0.016:
                argdel.append(i)
        if argdel:
            print('deleting from ridge', trench, ridge[argdel])
            ridge = np.delete(ridge, np.array(argdel))
            arggreat_dv = np.delete(arggreat_dv, np.array(argdel))

    dv_ridge = dvph2[arggreat_dv]
    if args.plot_age:
        agefld = step.fields['age'][0, :, :, 0]
        age_surface = np.ma.masked_where(agefld[:, indsurf] < 0.00001,
                                         agefld[:, indsurf])
        age_surface_dim = age_surface * vrms_surface *\
            args.ttransit / args.yearins / 1.e6
        agetrench = age_surface_dim[argless_dv]  # age at the trench
    else:
        agetrench = np.zeros(len(argless_dv))

    # writing the output into a file, all time steps are in one file
    for itrench in np.arange(len(trench)):
        fids[0].write("%7.0f %11.7f %10.6f %9.2f %9.2f \n" % (
            step.isnap,
            step.geom.ti_ad,
            trench[itrench],
            velocity_trench[itrench],
            agetrench[itrench]
        ))

    return trench, ridge, agetrench, dv_trench, dv_ridge


def plot_plate_limits(axis, ridges, trenches, ymin, ymax):
    """plot lines designating ridges and trenches"""
    for trench in trenches:
        axis.axvline(
            x=trench, ymin=ymin, ymax=ymax,
            color='red', ls='dashed', alpha=0.4)
    for ridge in ridges:
        axis.axvline(
            x=ridge, ymin=ymin, ymax=ymax,
            color='green', ls='dashed', alpha=0.4)
    axis.set_xlim(0, 2 * np.pi)
    axis.set_ylim(ymin, ymax)


def plot_plate_limits_field(axis, rcmb, ridges, trenches):
    """plot arrows designating ridges and trenches in 2D field plots"""
    for trench in trenches:
        xxd = (rcmb + 1.02) * np.cos(trench)  # arrow begin
        yyd = (rcmb + 1.02) * np.sin(trench)  # arrow begin
        xxt = (rcmb + 1.35) * np.cos(trench)  # arrow end
        yyt = (rcmb + 1.35) * np.sin(trench)  # arrow end
        axis.annotate('', xy=(xxd, yyd), xytext=(xxt, yyt),
                      arrowprops=dict(facecolor='red', shrink=0.05))
    for ridge in ridges:
        xxd = (rcmb + 1.02) * np.cos(ridge)
        yyd = (rcmb + 1.02) * np.sin(ridge)
        xxt = (rcmb + 1.35) * np.cos(ridge)
        yyt = (rcmb + 1.35) * np.sin(ridge)
        axis.annotate('', xy=(xxd, yyd), xytext=(xxt, yyt),
                      arrowprops=dict(facecolor='green', shrink=0.05))


def plot_plates(args, step, time, vrms_surface, trench, ridge, agetrench,
                topo, fids):
    """handle ploting stuff"""
    if step.sdat.par['boundaries']['air_layer']:
        dsa = step.sdat.par['boundaries']['air_thickness']
    else:
        dsa = 0.

    plt = args.plt
    lwd = args.linewidth
    vphi = step.fields['v2'][0, :, :, 0]
    tempfld = step.fields['T'][0, :, :, 0]
    concfld = step.fields['c'][0, :, :, 0]
    timestep = step.isnap

    if args.plot_age:
        agefld = step.fields['age'][0, :, :, 0]
    if args.plot_stress:
        stressfld = step.fields['sII'][0, :, :, 0]
        scale_stress = args.kappa * args.viscosity_ref / args.mantle**2

    if step.sdat.par['boundaries']['air_layer']:
        # we are a bit below the surface; delete "-some number"
        # to be just below
        # the surface (that is considered plane here); should check if you are
        # in the thermal boundary layer
        indsurf = np.argmin(abs((1 - dsa) - step.geom.r_coord)) - 4
        # depth to detect the continents
        indcont = np.argmin(abs((1 - dsa) - np.array(step.geom.r_coord))) - 10
    else:
        indsurf = -1
        indcont = -1  # depth to detect continents

    if step.sdat.par['boundaries']['air_layer'] and\
       not step.sdat.par['continents']['proterozoic_belts']:
        continents = np.ma.masked_where(
            np.logical_or(concfld[:-1, indcont] < 3,
                          concfld[:-1, indcont] > 4),
            concfld[:-1, indcont])
    elif (step.sdat.par['boundaries']['air_layer'] and
          step.sdat.par['continents']['proterozoic_belts']):
        continents = np.ma.masked_where(
            np.logical_or(concfld[:-1, indcont] < 3,
                          concfld[:-1, indcont] > 5),
            concfld[:-1, indcont])
    elif step.sdat.par['tracersin']['tracers_weakcrust']:
        continents = np.ma.masked_where(
            concfld[:-1, indcont] < 3, concfld[:-1, indcont])
    else:
        continents = np.ma.masked_where(
            concfld[:-1, indcont] < 2, concfld[:-1, indcont])

    # masked array, only continents are true
    continentsall = continents / continents

    if args.plot_age:
        age_surface = np.ma.masked_where(
            agefld[:, indsurf] < 0.00001, agefld[:, indsurf])
        age_surface_dim =\
            age_surface * vrms_surface * args.ttransit / args.yearins / 1.e6

    ph_coord = step.geom.p_coord

    # velocity
    vph2 = 0.5 * (vphi + np.roll(vphi, 1, 0))  # interpolate to the same phi
    dvph2 = (np.diff(vph2[:, indsurf]) / (ph_coord[0] * 2.))

    # plotting
    fig0, (ax1, ax2, ax3) = plt.subplots(3, 1, sharex=True, figsize=(12, 8))
    ax1.plot(ph_coord[:-1], concfld[:-1, indsurf],
             color='g', linewidth=lwd, label='Conc')
    ax2.plot(ph_coord[:-1], tempfld[:-1, indsurf],
             color='k', linewidth=lwd, label='Temp')
    ax3.plot(ph_coord[:-1], vph2[:-1, indsurf], linewidth=lwd, label='Vel')

    ax1.fill_between(
        ph_coord[:-1], continents, 1., facecolor='#8B6914', alpha=0.2)
    ax2.fill_between(
        ph_coord[:-1], continentsall, 0., facecolor='#8B6914', alpha=0.2)

    if step.sdat.par['boundaries']['topT_mode'] == 'iso':
        tempmin = step.sdat.par['boundaries']['topT_val'] * 0.9
    else:
        tempmin = 0.0
    if step.sdat.par['boundaries']['botT_mode'] == 'iso':
        tempmax = step.sdat.par['boundaries']['botT_val'] * 0.35
    else:
        tempmax = 0.8

    ax2.set_ylim(tempmin, tempmax)
    ax3.fill_between(
        ph_coord[:-1], continentsall * round(1.5 * np.amax(dvph2), 1),
        round(np.amin(dvph2) * 1.1, 1), facecolor='#8B6914', alpha=0.2)
    ax3.set_ylim(args.velocitymin, args.velocitymax)

    ax1.set_ylabel("Concentration", fontsize=args.fontsize)
    ax2.set_ylabel("Temperature", fontsize=args.fontsize)
    ax3.set_ylabel("Velocity", fontsize=args.fontsize)
    ax1.set_title(timestep, fontsize=args.fontsize)
    ax1.text(0.95, 1.07, str(round(time, 0)) + ' My',
             transform=ax1.transAxes, fontsize=args.fontsize)
    ax1.text(0.01, 1.07, str(round(step.geom.ti_ad, 8)),
             transform=ax1.transAxes, fontsize=args.fontsize)

    plot_plate_limits(ax3, ridge, trench, args.velocitymin, args.velocitymax)

    figname = misc.out_name(args, 'sveltempconc').format(timestep) + '.pdf'
    plt.savefig(figname, format='PDF')
    plt.close(fig0)

    # plotting velocity and velocity derivative
    fig0, (ax1, ax2) = plt.subplots(2, 1, sharex=True, figsize=(12, 8))
    ax1.plot(ph_coord[:-1], vph2[:-1, indsurf], linewidth=lwd, label='Vel')
    ax1.axhline(y=0, xmin=0, xmax=2 * np.pi,
                color='black', ls='solid', alpha=0.2)
    ax1.set_ylabel("Velocity", fontsize=args.fontsize)
    ax1.text(0.95, 1.07, str(round(time, 0)) + ' My',
             transform=ax1.transAxes, fontsize=args.fontsize)
    ax1.text(0.01, 1.07, str(round(step.geom.ti_ad, 8)),
             transform=ax1.transAxes, fontsize=args.fontsize)
    ax2.plot(ph_coord[:-1] + ph_coord[0], dvph2,
             color='k', linewidth=lwd, label='dv')
    ax2.set_ylabel("dv", fontsize=args.fontsize)

    plot_plate_limits(ax1, ridge, trench, args.velocitymin, args.velocitymax)
    plot_plate_limits(ax2, ridge, trench, args.dvelocitymin, args.dvelocitymax)
    ax1.set_xlim(0, 2 * np.pi)
    ax1.set_title(timestep, fontsize=args.fontsize)

    ax1.fill_between(
        ph_coord[:-1], continentsall * args.velocitymin, args.velocitymax,
        facecolor='#8b6914', alpha=0.2)
    ax1.set_ylim(args.velocitymin, args.velocitymax)
    ax2.fill_between(
        ph_coord[:-1], continentsall * args.dvelocitymin, args.dvelocitymax,
        facecolor='#8b6914', alpha=0.2)
    ax2.set_ylim(args.dvelocitymin, args.dvelocitymax)

    figname = misc.out_name(args, 'sveldvel').format(timestep) + '.pdf'
    plt.savefig(figname, format='PDF')
    plt.close(fig0)

    # plotting velocity and second invariant of stress
    if args.plot_stress:
        fig0, (ax1, ax2) = plt.subplots(2, 1, sharex=True, figsize=(12, 8))
        ax1.plot(ph_coord[:-1], vph2[:-1, indsurf], linewidth=lwd, label='Vel')
        ax1.axhline(y=0, xmin=0, xmax=2 * np.pi,
                    color='black', ls='solid', alpha=0.2)
        ax1.set_ylabel("Velocity", fontsize=args.fontsize)
        ax1.text(0.95, 1.07, str(round(time, 0)) + ' My',
                 transform=ax1.transAxes, fontsize=args.fontsize)
        ax1.text(0.01, 1.07, str(round(step.geom.ti_ad, 8)),
                 transform=ax1.transAxes, fontsize=args.fontsize)
        ax2.plot(ph_coord[:-1], stressfld[:-1, indsurf] * scale_stress / 1.e6,
                 color='k', linewidth=lwd, label='Stress')
        ax2.set_ylim(args.stressmin, args.stressmax)
        ax2.set_ylabel("Stress [MPa]", fontsize=args.fontsize)

        plot_plate_limits(ax1, ridge, trench,
                          args.velocitymin, args.velocitymax)
        plot_plate_limits(ax2, ridge, trench,
                          args.stressmin, args.stressmax)
        ax1.set_xlim(0, 2 * np.pi)
        ax1.set_title(timestep, fontsize=args.fontsize)

        ax1.fill_between(
            ph_coord[:-1], continentsall * args.velocitymin, args.velocitymax,
            facecolor='#8B6914', alpha=0.2)
        ax1.set_ylim(args.velocitymin, args.velocitymax)
        ax2.fill_between(
            ph_coord[:-1], continentsall * args.dvelocitymin,
            args.dvelocitymax,
            facecolor='#8B6914', alpha=0.2)

        figname = misc.out_name(args, 'svelstress').format(timestep) + '.pdf'
        plt.savefig(figname, format='PDF')
        plt.close(fig0)

    # plotting velocity
    fig1, (ax1, ax2) = plt.subplots(2, 1, sharex=True, figsize=(12, 8))
    ax1.plot(ph_coord[:-1], vph2[:-1, indsurf], linewidth=lwd, label='Vel')
    ax1.axhline(y=0, xmin=0, xmax=2 * np.pi,
                color='black', ls='solid', alpha=0.2)
    ax1.set_ylim(args.velocitymin, args.velocitymax)
    ax1.set_ylabel("Velocity", fontsize=args.fontsize)
    ax1.text(0.95, 1.07, str(round(time, 0)) + ' My',
             transform=ax1.transAxes, fontsize=args.fontsize)
    plot_plate_limits(ax1, ridge, trench, args.velocitymin, args.velocitymax)

    # plotting velocity and age at surface
    if args.plot_age:
        fig2, (ax3, ax4) = plt.subplots(2, 1, sharex=True, figsize=(12, 8))
        ax3.plot(ph_coord[:-1], vph2[:-1, indsurf], linewidth=lwd, label='Vel')
        ax3.axhline(
            y=0, xmin=0, xmax=2 * np.pi,
            color='black', ls='solid', alpha=0.2)
        ax3.set_ylim(args.velocitymin, args.velocitymax)
        ax3.set_ylabel("Velocity", fontsize=args.fontsize)
        ax3.text(0.95, 1.07, str(round(time, 0)) + ' My',
                 transform=ax3.transAxes, fontsize=args.fontsize)
        ax3.fill_between(
            ph_coord[:-1], continentsall * args.velocitymax, args.velocitymin,
            facecolor='#8B6914', alpha=0.2)
        plot_plate_limits(ax3, ridge, trench,
                          args.velocitymin, args.velocitymax)

    times_subd = []
    age_subd = []
    distance_subd = []
    ph_trench_subd = []
    ph_cont_subd = []
    if step.sdat.par['switches']['cont_tracers']:
        for i in range(len(trench)):  # should enumerate
            # detection of the distance in between subduction and continent
            ph_coord_noend = ph_coord[:-1]
            angdistance1 = abs(ph_coord_noend[continentsall == 1] - trench[i])
            angdistance2 = 2. * np.pi - angdistance1
            angdistance = np.minimum(angdistance1, angdistance2)
            distancecont = min(angdistance)
            argdistancecont = np.argmin(angdistance)
            continentpos = ph_coord_noend[continentsall == 1][argdistancecont]

            ph_trench_subd.append(trench[i])
            age_subd.append(agetrench[i])
            ph_cont_subd.append(continentpos)
            distance_subd.append(distancecont)
            times_subd.append(step.geom.ti_ad)

            if angdistance1[argdistancecont] < angdistance2[argdistancecont]:
                if continentpos - trench[i] < 0:  # continent is on the left
                    distancecont = - distancecont
                ax1.annotate('', xy=(trench[i] + distancecont, 2000),
                             xycoords='data', xytext=(trench[i], 2000),
                             textcoords='data',
                             arrowprops=dict(arrowstyle="->", lw="2",
                                             shrinkA=0, shrinkB=0))
            else:  # distance over boundary
                xy_anot, xy_text = 0, 2 * np.pi
                if continentpos - trench[i] < 0:
                    xy_anot, xy_text = xy_text, xy_anot
                ax1.annotate('', xy=(xy_anot, 2000),
                             xycoords='data', xytext=(trench[i], 2000),
                             textcoords='data',
                             arrowprops=dict(arrowstyle="-", lw="2",
                                             shrinkA=0, shrinkB=0))
                ax1.annotate('', xy=(continentpos, 2000),
                             xycoords='data', xytext=(xy_text, 2000),
                             textcoords='data',
                             arrowprops=dict(arrowstyle="->", lw="2",
                                             shrinkA=0, shrinkB=0))

    ax1.fill_between(
        ph_coord[:-1], continentsall * args.velocitymin, args.velocitymax,
        facecolor='#8B6914', alpha=0.2)
    ax2.set_ylabel("Topography [km]", fontsize=args.fontsize)
    ax2.axhline(y=0, xmin=0, xmax=2 * np.pi,
                color='black', ls='solid', alpha=0.2)
    ax2.plot(topo[:, 0],
             topo[:, 1] * args.mantle / 1.e3,
             color='black')
    ax2.set_xlim(0, 2 * np.pi)
    ax2.set_ylim(args.topomin, args.topomax)
    ax2.fill_between(
        ph_coord[:-1], continentsall * args.topomax, args.topomin,
        facecolor='#8B6914', alpha=0.2)
    plot_plate_limits(ax2, ridge, trench, args.topomin, args.topomax)
    ax1.set_title(timestep, fontsize=args.fontsize)
    figname = misc.out_name(args, 'sveltopo').format(timestep) + '.pdf'
    fig1.savefig(figname, format='PDF')
    plt.close(fig1)

    if args.plot_age:
        ax4.set_ylabel("Seafloor age [My]", fontsize=args.fontsize)
        # in dimensions
        ax4.plot(ph_coord[:-1], age_surface_dim[:-1], color='black')
        ax4.set_xlim(0, 2 * np.pi)
        ax4.fill_between(
            ph_coord[:-1], continentsall * args.agemax, args.agemin,
            facecolor='#8B6914', alpha=0.2)
        ax4.set_ylim(args.agemin, args.agemax)
        plot_plate_limits(ax4, ridge, trench, args.agemin, args.agemax)
        ax3.set_title(timestep, fontsize=args.fontsize)
        figname = misc.out_name(args, 'svelage').format(timestep) + '.pdf'
        fig2.savefig(figname, format='PDF')
        plt.close(fig2)

    # writing the output into a file, all time steps are in one file
    for isubd in np.arange(len(distance_subd)):
        fids[1].write("%6.0f %11.7f %11.3f %10.6f %10.6f %10.6f %11.3f\n" % (
            timestep,
            times_subd[isubd],
            time,
            distance_subd[isubd],
            ph_trench_subd[isubd],
            ph_cont_subd[isubd],
            age_subd[isubd],
        ))


def io_surface(timestep, time, fid, fld):
    """Output for surface files"""
    fid.write("{} {}".format(timestep, time))
    fid.writelines(["%10.2e" % item for item in fld[:]])
    fid.writelines(["\n"])


def lithospheric_stress(args, step, trench, ridge, time):
    """calculate stress in the lithosphere"""
    timestep = step.isnap
    lwd = args.linewidth
    base_lith = step.geom.rcmb + 1 - 0.105
    scale_stress = args.kappa * args.viscosity_ref / args.mantle**2
    scale_dist = args.mantle

    stressfld = step.fields['sII'][0, :, :, 0]
    stressfld = np.ma.masked_where(step.geom.r_mesh[0] < base_lith, stressfld)

    # stress integration in the lithosphere
    dzm = (step.geom.r_coord[1:] - step.geom.r_coord[:-1])
    stress_lith = np.sum((stressfld[:, 1:] * dzm.T), axis=1)
    ph_coord = step.geom.p_coord  # probably doesn't need alias

    # plot stress in the lithosphere
    fig, axis = args.plt.subplots(ncols=1)
    surf = axis.pcolormesh(step.geom.x_mesh[0], step.geom.y_mesh[0],
                           stressfld * scale_stress / 1.e6,
                           cmap='gnuplot2_r',
                           rasterized=not args.pdf, shading='gouraud')
    surf.set_clim(vmin=0, vmax=300)
    cbar = args.plt.colorbar(surf, shrink=args.shrinkcb)
    cbar.set_label(r'${}$'.format(constants.FIELD_VARS['sII'].shortname))
    args.plt.axis('equal')
    args.plt.axis('off')
    # Annotation with time and step
    axis.text(1., 0.9, str(round(time, 0)) + ' My',
              transform=axis.transAxes, fontsize=args.fontsize)
    axis.text(1., 0.1, str(timestep),
              transform=axis.transAxes, fontsize=args.fontsize)
    args.plt.savefig(
        misc.out_name(args, 'lith').format(timestep) + '.pdf',
        format='PDF')
    args.plt.close(fig)

    # velocity
    vphi = step.fields['v2'][0, :, :, 0]
    vph2 = 0.5 * (vphi + np.roll(vphi, 1, 0))  # interpolate to the same phi

    # position of continents
    concfld = step.fields['c'][0, :, :, 0]
    if step.sdat.par['boundaries']['air_layer']:
        # we are a bit below the surface; delete "-some number"
        # to be just below
        dsa = step.sdat.par['boundaries']['air_thickness']
        # depth to detect the continents
        indcont = np.argmin(abs((1 - dsa) - step.geom.r_coord)) - 10
    else:
        # depth to detect continents
        indcont = -1
    if step.sdat.par['boundaries']['air_layer'] and\
            not step.sdat.par['continents']['proterozoic_belts']:
        continents = np.ma.masked_where(
            np.logical_or(concfld[:-1, indcont] < 3,
                          concfld[:-1, indcont] > 4),
            concfld[:-1, indcont])
    elif step.sdat.par['boundaries']['air_layer'] and\
            step.sdat.par['continents']['proterozoic_belts']:
        continents = np.ma.masked_where(
            np.logical_or(concfld[:-1, indcont] < 3,
                          concfld[:-1, indcont] > 5),
            concfld[:-1, indcont])
    elif step.sdat.par['tracersin']['tracers_weakcrust']:
        continents = np.ma.masked_where(
            concfld[:-1, indcont] < 3, concfld[:-1, indcont])
    else:
        continents = np.ma.masked_where(
            concfld[:-1, indcont] < 2, concfld[:-1, indcont])

    # masked array, only continents are true
    continentsall = continents / continents

    # plot integrated stress in the lithosphere
    fig0, (ax1, ax2) = args.plt.subplots(2, 1, sharex=True, figsize=(12, 8))
    ax1.plot(ph_coord[:-1], vph2[:-1, -1], linewidth=lwd, label='Vel')
    ax1.axhline(y=0, xmin=0, xmax=2 * np.pi,
                color='black', ls='solid', alpha=0.2)
    ax1.set_ylabel("Velocity", fontsize=args.fontsize)
    ax1.text(0.95, 1.07, str(round(time, 0)) + ' My',
             transform=ax1.transAxes, fontsize=args.fontsize)
    ax1.text(0.01, 1.07, str(round(step.geom.ti_ad, 8)),
             transform=ax1.transAxes, fontsize=args.fontsize)

    ax2.plot(ph_coord, stress_lith * scale_stress * scale_dist / 1.e12,
             color='k', linewidth=lwd, label='Stress')
    ax2.set_ylabel(r"Integrated stress [$TN\,m^{-1}$]", fontsize=args.fontsize)

    plot_plate_limits(ax1, ridge, trench, args.velocitymin, args.velocitymax)
    plot_plate_limits(ax2, ridge, trench, args.stressmin, args.lstressmax)
    ax1.set_xlim(0, 2 * np.pi)
    ax1.set_title(timestep, fontsize=args.fontsize)

    ax1.fill_between(
        ph_coord[:-1], continentsall * args.velocitymin, args.velocitymax,
        facecolor='#8b6914', alpha=0.2)
    ax1.set_ylim(args.velocitymin, args.velocitymax)
    ax2.fill_between(
        ph_coord[:-1], continentsall * args.stressmin, args.lstressmax,
        facecolor='#8b6914', alpha=0.2)
    ax2.set_ylim(args.stressmin, args.lstressmax)

    figname = misc.out_name(args, 'svelslith').format(timestep) + '.pdf'
    fig0.savefig(figname, format='PDF')
    args.plt.close(fig0)


def plates_cmd(args):
    """find positions of trenches and subductions

    uses velocity field (velocity derivation)
    plots the number of plates over a designated lapse of time
    """
    sdat = StagyyData(args.path)
    if not args.vzcheck:
        # calculating averaged horizontal surface velocity
        # needed for redimensionalisation
        ilast = sdat.rprof.index.levels[0][-1]
        rlast = sdat.rprof.loc[ilast]
        nprof = 0
        uprof_averaged = rlast.loc[:, 'vhrms'] * 0
        for step in misc.steps_gen(sdat, args):
            if step.rprof is None:
                continue
            uprof_averaged += step.rprof['vhrms']
            nprof += 1
        uprof_averaged /= nprof
        radius = rlast['r'].values
        if sdat.par['boundaries']['air_layer']:
            dsa = sdat.par['boundaries']['air_thickness']
            isurf = np.argmin(abs(radius - radius[-1] + dsa))
        else:
            isurf = -1
        vrms_surface = uprof_averaged.iloc[isurf]
        if sdat.par['boundaries']['air_layer']:
            isurf = np.argmin(abs((1 - dsa) - radius))
            isurf -= 4  # why different isurf for the rest?

        with misc.InchoateFiles(8, 'plates') as fids:
            fids.fnames = ['plate_velocity', 'distance_subd', 'continents',
                           'flux', 'topography', 'age', 'velderiv', 'velocity']
            fids[0].write('#  it  time  ph_trench vel_trench age_trench\n')
            fids[1].write('#  it      time   time [My]   distance     '
                          'ph_trench     ph_cont  age_trench [My]\n')

            istart, iend = None, None
            for step in misc.steps_gen(sdat, args):
                # could check other fields too
                if step.fields['T'] is None:
                    continue
                timestep = step.isnap
                istart = timestep if istart is None else istart
                iend = timestep
                print('Treating snapshot', timestep)

                rcmb = step.geom.rcmb
                if args.plot_stress:
                    scale_stress = args.kappa * \
                        args.viscosity_ref / args.mantle**2

                # topography
                fname = sdat.filename('sc', timestep=timestep, suffix='.dat')
                topo = np.genfromtxt(str(fname))
                # rescaling topography!
                if sdat.par['boundaries']['air_layer']:
                    topo[:, 1] = topo[:, 1] / (1. - dsa)

                time = step.geom.ti_ad * vrms_surface *\
                    args.ttransit / args.yearins / 1.e6
                trenches, ridges, agetrenches, _, _ =\
                    detect_plates(args, step, vrms_surface, fids, time)
                plot_plates(args, step, time, vrms_surface, trenches, ridges,
                            agetrenches, topo, fids)

                # prepare for continent plotting
                concfld = step.fields['c'][0, :, :, 0]
                continentsfld = np.ma.masked_where(
                    concfld < 3, concfld)  # plotting continents, to-do
                continentsfld = continentsfld / continentsfld

                temp = step.fields['T'][0, :, :, 0]
                tgrad = (temp[:, isurf - 1] - temp[:, isurf]) /\
                    (step.geom.r_coord[isurf] - step.geom.r_coord[isurf - 1])

                io_surface(timestep, time, fids[2], concfld[:-1, isurf])
                io_surface(timestep, time, fids[3], tgrad)
                io_surface(timestep, time, fids[4], topo[:, 1])
                if args.plot_age:
                    io_surface(timestep, time, fids[5],
                               step.fields['age'][0, :, isurf, 0])

                # plot viscosity field with position of trenches and ridges
                fig, axis, surf, _ = field.plot_scalar(args, step, 'eta')
                etamax = sdat.par['viscosity']['eta_max']
                surf.set_clim(vmin=1e-2, vmax=etamax)

                # plotting continents
                xmesh, ymesh = step.geom.x_mesh[0], step.geom.y_mesh[0]
                axis.pcolormesh(xmesh, ymesh, continentsfld,
                                rasterized=not args.pdf, cmap='cool_r',
                                vmin=0, vmax=0, shading='goaround')
                cmap2 = args.plt.cm.ocean
                cmap2.set_over('m')

                # plotting velocity vectors
                field.plot_vec(axis, step, 'v')

                # Annotation with time and step
                axis.text(1., 0.9, str(round(time, 0)) + ' My',
                          transform=axis.transAxes, fontsize=args.fontsize)
                axis.text(1., 0.1, str(timestep),
                          transform=axis.transAxes, fontsize=args.fontsize)

                # Put arrow where ridges and trenches are
                plot_plate_limits_field(axis, rcmb, ridges, trenches)

                # Save figure
                args.plt.tight_layout()
                args.plt.savefig(
                    misc.out_name(args, 'eta').format(timestep) + '.pdf',
                    format='PDF')

                # Zoom
                if args.zoom is not None:
                    if args.zoom > 360 or args.zoom < 0:
                        print(' *WARNING* ')
                        print('Zoom angle should be positive and '
                              'less than 360 deg')
                        print(' Exiting the code ')
                        sys.exit()
                    if args.zoom > 315. or args.zoom <= 45:
                        ladd, radd, uadd, dadd = 0.1, 0.05, 0.8, 0.8
                    elif args.zoom > 45. and args.zoom <= 135:
                        ladd, radd, uadd, dadd = 0.8, 0.8, 0.05, 0.1
                    elif args.zoom > 135. and args.zoom <= 225:
                        ladd, radd, uadd, dadd = 0.05, 0.1, 0.8, 0.8
                    else:
                        ladd, radd, uadd, dadd = 0.8, 0.8, 0.1, 0.05
                    xzoom = (rcmb + 1) * np.cos(args.zoom / 180. * np.pi)
                    yzoom = (rcmb + 1) * np.sin(args.zoom / 180. * np.pi)
                    axis.set_xlim(xzoom - ladd, xzoom + radd)
                    axis.set_ylim(yzoom - dadd, yzoom + uadd)
                    args.plt.savefig(
                        misc.out_name(args, 'etazoom').format(timestep) +
                        '.pdf', format='PDF')
                args.plt.close(fig)

                # plot stress field with position of trenches and ridges
                if args.plot_stress:
                    fig, axis, surf, cbar = field.plot_scalar(
                        args, step, 'sII', scale_stress / 1.e6)
                    surf.set_clim(vmin=0, vmax=300)
                    cbar.set_label('Stress [MPa]')

                    # Annotation with time and step
                    axis.text(1., 0.9, str(round(time, 0)) + ' My',
                              transform=axis.transAxes, fontsize=args.fontsize)
                    axis.text(1., 0.1, str(timestep),
                              transform=axis.transAxes, fontsize=args.fontsize)

                    # Put arrow where ridges and trenches are
                    plot_plate_limits_field(axis, rcmb, ridges, trenches)

                    # Save figure
                    args.plt.tight_layout()
                    args.plt.savefig(
                        misc.out_name(args, 's').format(timestep) + '.pdf',
                        format='PDF')

                    # Zoom
                    if args.zoom is not None:
                        axis.set_xlim(xzoom - ladd, xzoom + radd)
                        axis.set_ylim(yzoom - dadd, yzoom + uadd)
                        args.plt.savefig(
                            misc.out_name(args, 'szoom').format(timestep) +
                            '.pdf', format='PDF')
                    args.plt.close(fig)

                    # calculate stresses in the lithosphere
                    lithospheric_stress(args, step, trenches, ridges, time)

                # plotting the principal deviatoric stress field
                if args.plot_deviatoric_stress:
                    fig, axis, _, _ = field.plot_scalar(args, step, 'sII',
                                                        alpha=0.1)

                    # plotting continents
                    axis.pcolormesh(xmesh, ymesh, continentsfld,
                                    rasterized=not args.pdf, cmap='cool_r',
                                    vmin=0, vmax=0, shading='goaround')
                    cmap2 = args.plt.cm.ocean
                    cmap2.set_over('m')

                    # plotting principal deviatoric stress
                    field.plot_vec(axis, step, 'sx')

                    # Annotation with time and step
                    axis.text(1., 0.9, str(round(time, 0)) + ' My',
                              transform=axis.transAxes, fontsize=args.fontsize)
                    axis.text(1., 0.1, str(timestep),
                              transform=axis.transAxes, fontsize=args.fontsize)

                    # Put arrow where ridges and trenches are
                    plot_plate_limits_field(axis, rcmb, ridges, trenches)

                    args.plt.tight_layout()
                    args.plt.savefig(
                        misc.out_name(args, 'sx').format(timestep) + '.pdf',
                        format='PDF')
                    args.plt.close(fig)

            # determine names of files
            stem = '{}_{}_{}'.format(fids.fnames[0], istart, iend)
            idx = 0
            fmt = '{}.dat'
            while os.path.isfile(fmt.format(stem, idx)):
                fmt = '{}_{}.dat'
                idx += 1
            fids.fnames = [fmt.format('{}_{}_{}'.format(fname, istart, iend),
                                      idx)
                           for fname in fids.fnames]
    else:  # args.vzcheck
        seuil_memz = 0
        nb_plates = []
        time = []
        ch2o = []
        istart, iend = None, None

        for step in misc.steps_gen(sdat, args):
            # could check other fields too
            if step.fields['T'] is None:
                continue
            if args.timeprofile:
                time.append(step.timeinfo.loc['t'])
                ch2o.append(step.timeinfo.loc[0])
            istart = step.isnap if istart is None else istart
            iend = step.isnap
            plt = args.plt
            limits, nphi, dvphi, seuil_memz, vphi_surf, water_profile =\
                detect_plates_vzcheck(step, seuil_memz)
            limits.sort()
            sizeplates = [limits[0] + nphi - limits[-1]]
            for lim in range(1, len(limits)):
                sizeplates.append(limits[lim] - limits[lim - 1])
            lim = len(limits) * [max(dvphi)]
            plt.figure(step.istep)
            plt.subplot(221)
            plt.axis([0, nphi,
                      np.min(vphi_surf) * 1.2, np.max(vphi_surf) * 1.2])
            plt.plot(vphi_surf)
            plt.subplot(223)
            plt.axis(
                [0, nphi,
                 np.min(dvphi) * 1.2, np.max(dvphi) * 1.2])
            plt.plot(dvphi)
            plt.scatter(limits, lim, color='red')
            plt.subplot(222)
            plt.hist(sizeplates, 10, (0, nphi / 2))
            plt.subplot(224)
            plt.plot(water_profile)
            plt.savefig('plates' + str(step.isnap) + '.pdf', format='PDF')
            plt.close(step.istep)

            nb_plates.append(len(limits))

        if args.timeprofile:
            for i in range(2, len(nb_plates) - 3):
                nb_plates[i] = (nb_plates[i - 2] + nb_plates[i - 1] +
                                nb_plates[i] + nb_plates[i + 1] +
                                nb_plates[i + 2]) / 5
            plt.figure(-1)
            plt.subplot(121)
            plt.axis([time[0], time[-1], 0, np.max(nb_plates)])
            plt.plot(time, nb_plates)
            plt.subplot(122)
            plt.plot(time, ch2o)
            plt.savefig('plates_{}_{}.pdf'.format(istart, iend), format='PDF')
            plt.close(-1)



"""StagYY par file handling"""

import f90nml
import sys
from .constants import CONFIG_DIR

PAR_DFLT_FILE = CONFIG_DIR / 'par'
PAR_DEFAULT = {
    'switches': {
        'verbose': False,
        'try_continue': False,
        'compressible': False,
        'dens_strat': True,
        'dens_actual': False,
        'melting': False,
        'composition': False,
        'phase_changes': False,
        'geoid_kernel_mode': False,
        'tracers': False,
        'cont_rafts': False,
        'cont_tracers': False,
        'clb_experiment': False,
        'chem_bm1': False,
        'pvk_bm': False,
        'Io_tidal_heating': False,
        'atmosphere': False,
        'multig_solve': True,
        'plates_analyse': False,
        'dimensional_units': False,
        'total_pressure': False,
        'dens_read_Fabio': False,
        'visc_read_Fabio': False,
        'quiet': False,
    },

    'geometry': {
        'shape': 'spherical',
        'nxtot': 1,
        'nytot': 512,
        'nztot': 64,
        'aspect_ratio': [1.0, 10.0, 1.0],
        'theta_position': 'default',
        'zspacing_mode': 'constant',
        'dresl_topbot': 1.0,
        'D_dimensional': 2890.e3,
        'D_dim_top': 0.0e3,
        'anti_squeeze': True,
        'anti_squeeze_r_fraction': 0.5,
        'dresl_air': 1.0,
        'dresl_surf': 1.0,
        'dresl_cmb': 1.0,
        'dresl_ppv': 1.0,
        'dresl_660': 1.0,
        'dresl_400': 1.0,
        'wresl_air': 0.1,
        'wresl_surf': 0.1,
        'wresl_cmb': 0.1,
        'wresl_ppv': 0.1,
        'wresl_660': 0.1,
        'wresl_400': 0.1,
        'r_cmb': 1.19,
        'yy_MinOverlap': True,
        'kmaxb': 0.25,
        'kmaxt': 0.25,
        'npbl': 10,
    },

    'refstate': {
        'Di0': 1.18,
        'ra0': 1.e5,
        'nradiogenic': 1,
        'Rh': [0.0],
        'Rh_tstart': 0.,
        'drho_thermal': 0.125,
        'perplex_properties': False,
        'perplex_hz_file': 'harzburgite.dat',
        'perplex_bs_file': 'basalt.dat',
        'perplex_pr_file': 'primordial.dat',
        'perplex_py_file': 'pyrolite.dat',
        'perplex_dT': 100.,
        'Tref_surf': 0.64,
        'rhoref_surf': [3240., 3080., 2900., 2900., 3080., 3080., 7000.],
        'gr_s': [1.3],
        'drho_surf': 1.0,
        'drho_cmb': 4337.,
        'q_gamma': [1.0],
        'delT0_expan': [6.0],
        'ex_tkappa': [3.0],
        'ak_expan': [1.4],
        'c_dependent_heating': False,
        'dHeat_dprim': 1.0,
        'dHeat_dbasalt': 1.0,
        'Rh_halflife': ['BIG'],
        'deltaT_dimensional': 2500.0,
        'g_dimensional': 9.81,
        'gravity_3D': False,
        'core_superheat': 0.0,
        'tcond_dimensional': 3.0,
        'core_tcond': 46.0,
        'dens_dimensional': 3300.,
        'expan_dimensional': 3.e-5,
        'Cp_dimensional': 1200.,
        'core_cooling': False,
        'core_model': 'simple',
        'core_Kppm': 0.0,
        'core_Tmelt0': 5600,
        'core_heatcap': 6.38e12,
        'phase_assemblage_varies_with_C': False,
        'air_density': 1.e-3,
        'topregion_adiabatic': False,
        'topregion_z': 1.0,
        'topregion_thalf': 'BIG',
        'Birch_Murnaghan': False,
        'K0': [210.e9],
        'Kp': [3.9],
        'heterogeneous_heating': False,
        'Hs_zbot': 0.,
        'Hs_ztop': 1.,
        'Hs_bot': 0.,
        'Hs_top': 0.,
        'Hs_mode': 1,
    },

    'boundaries': {
        'topT_mode': 'isothermal',
        'topT_val': 1.0,
        'botT_mode': 'iso',
        'botT_val': 0.0,
        'outT_val': -1.0,
        'topV_mode': 'free-slip',
        'botV_mode': 'free-slip',
        'x_patch': 0.5,
        'y_patch': 0.5,
        'r_patch': 0.1,
        'x_spread': 0.0,
        'v_spread': 0.0,
        'nx_plates': 2,
        'ny_plates': 2,
        'x_bc': 'ww',
        'y_bc': 'ww',
        'v_mantle': 0.0,
        'air_layer': False,
        'air_thickness': 0.1,
        'air_threshold': 0.8,
        'virtual_bndry_distance': -1.,
        'zero_surface_mean_flow': True,
        'Tsurf_eqm': 300.,
        'topT_locked_subsolar': 300.,
        'topT_locked_farside': 200.,
        'vbc_file_stem': 'vbc',
        'plateid_file_stem': 'pid',
        'vbc_file_interval_Myr': 1.0,
        'vbc_file_start_Myr': -70.,
        'platerecon_time_origin': 0.0,
        'platerecon_time_init': 0.0,
        'platerecon_time_end': 0.0,
        'BottomPhaseChange': False,
        'BotPphase': 1.,
        'TopPhaseChange': False,
        'TopPphase': 1.,
    },

    't_init': {
        'imode_t': 'cells',
        'amp_t': 0.00,
        'blthick': 0.1,
        't0_init': 0.5,
        'z0_t': 0.5,
        'w0_t': 0.5,
        'dip': 0.0,
        'length': 0.0,
        'ocean_age': 60.0,
        'cont_thick': 100.e3,
    },

    'timein': {
        'nsteps': 1,
        'nwrite': 1,
        'alpha_adv': 0.5,
        'alpha_diff': 1.0,
        'iord': 2,
        'advection_scheme': 'MPDATA',
        'stoptime': 'hhmm',
        'dt_write': 'BIG',
        'finish_at_time': 'BIG',
        'TVD_limiter': 2.,
        'diffusion_implicit': False,
        'diffusion_beta': 1.0,
        'diffusion_errmax': 1.e-4,
        'diffusion_maxits': 100,
        'diffusion_alpha': 0.6,
        'diffusion_nrelax': 3,
        'density_jump_stabilization': False,
        'density_jump_theta': 1.0,
        'density_jump_simplify': True,
        'PBSTimeRequested': 168.0,
    },

    'viscosity': {
        'ietalaw': 0,
        'E_eta': [0.0],
        'V_eta': [0.0],
        'Pdecay_V': ['BIG'],
        'Ts_eta': -1.0,
        'n_eta': 3.5,
        'eta0': 1.0,
        'T0_eta': 0.5,
        'd0_eta': 0.5,
        'stress0_eta': [1.0],
        'deta_p': [1.0],
        'deta_p_correct_jumps': False,
        'diffusion_creep': True,
        'dislocation_creep': False,
        'eta_max': 1.e5,
        'eta_min': 1.e-5,
        'eta_minmax_depthmax': 'BIG',
        'deta_basalt': 1.0,
        'deta_ccrust': 1.0,
        'deta_primordial': 1.0,
        'deta_metal': 1.0,
        'deta_water': 1.0,
        'etalaw_basalt': 1,
        'etalaw_ccrust': 1,
        'etalaw_metal': 1,
        'etalaw_melt': 1,
        'etalaw_primordial': 1,
        'etalaw_water': 1,
        'c0eta_basalt': 0.0,
        'c0eta_ccrust': 0.0,
        'c0eta_primordial': 0.0,
        'c0eta_metal': 0.0,
        'c0eta_water': 0.0,
        'weak_line': False,
        'weak_dx': 0.1,
        'weak_dz': 0.05,
        'weak_eta': 1.0,
        'strong_cont_rafts': False,
        'strong_eta': 1.0,
        'strong_dz': 0.05,
        'vertonly_eta': False,
        'horzonly_eta': False,
        'plasticity': False,
        'c_dependent_yieldpars': False,
        'stressY_eta': 'BIG',
        'dstressY_eta_dp': 0.0,
        'cohesion_eta': 'BIG',
        'frictionCoeff_eta': 'BIG',
        'stressY_basalt': 'BIG',
        'dstressY_basalt': 0.0,
        'cohesion_basalt': 'BIG',
        'frictionCoeff_basalt': 'BIG',
        'stressY_ccrust': 'BIG',
        'dstressY_ccrust': 0.0,
        'cohesion_ccrust': 'BIG',
        'frictionCoeff_ccrust': 'BIG',
        'stressY_metal': 'BIG',
        'dstressY_metal': 0.0,
        'cohesion_metal': 'BIG',
        'frictionCoeff_metal': 'BIG',
        'stressY_primordial': 'BIG',
        'dstressY_primordial': 0.0,
        'cohesion_primordial': 'BIG',
        'frictionCoeff_primordial': 'BIG',
        'dstressY_eta_water': 1.0,
        'water_dependent_stressY_eta': False,
        'd_etalaw': 1,
        'Rheal_law': 'constant',
        'E_Rh': 0.0,
        'F_eta': 0.0,
        'eta_melt': False,
        'deta_melt': 1.0,
        'mean_T_eta': False,
        'T0_Tmean': 0.5,
        'E_eta_Tmean': 0.0,
        'eta_conv': 0.01,
        'alpha_eta': 1.0,
        'maxits_eta': 20,
        'bmoore_eta': False,
        'aniso_ep': False,
        'visc_interp_law': 'g',
        'phase_dep_Eeta': False,
        'eta_air': 1.,
        'PressureDepRheology': False,
        'PressureDepVact': False,
        'RaFromRheolPref': False,
    },

    'grainsize': {
        'graingrowth_k': 1.,
        'graingrowth_p': 4.,
        'graingrowth_E': 300.e3,
        'grainreduction_C': 1.,
        'grainsize0_eta': 1.,
        'grainsize_n_eta': 3.0,
        'GS_ForcedPiezometric': True,
        'GS_TwoPhases': False,
        'GS_PiezometerModel': 'homogeneous',
        'GG_k0': [2.49813e7, 2.49813e7],
        'GG_p': [2.0, 2.0],
        'GG_E': [2.e5, 2.e5],
        'GS_InitSize': [1e2, 1e2],
        'GS_gamma': [1e6, 1e6],
        'curv_k0': 74943.9,
        'curv_E': 2.e5,
        'curv_q': 1.5,
        'curv_gamma': 1.e6,
        'curv_Init': 1e2,
        'GSD_sigma': 0.8,
        'GS_fG0': 1.e-2,
        'GS_fI0': 0.0,
        'transition_depth_GS': [410.e3, 500.e3, 660.e3, 2740.e3],
        'GSAfterPhaseTrans': 5.e0,
        'curvDownThrough660': 5.e0,
    },

    'iteration': {
        'maxits': 50,
        'minits': 1,
        'errmax': 1.e-3,
        'alpha_mom': 0.7,
        'alpha_cont': 0.9,
        'redblack': False,
        'extra_lid_relax': False,
        'nlid_relax': 1,
        'tlid': 0.2,
        'extra_hiresid_relax': False,
        'nhi_relax': 1,
        'normalize_res_by_eta': False,
        'relax_kernel': 'classic',
    },

    'multi': {
        'maxcoarseits': 200,
        'nrelax1': 3,
        'nrelax2': 3,
        'nhmin': 4,
        'nzmin': 4,
        'nzpnmin': 1,
        'alpha_multi': 1.0,
        'matrix_dep_pinterp': True,
        'matrix_dep_vinterp': False,
        'ncpus_per_node': 8,
        'nppcpu_opt_samenode': 64,
        'nppcpu_opt_xnode': 1024,
        'more_coarse_its': True,
        'f_cycle': False,
        'direct_coarse_soln': False,
        'idle_duplicates': True,
    },

    'ioin': {
        'input_file': 'null',
        'input_frame': 1,
        'output_file_stem': 'test',
        't_write': True,
        'vp_write': False,
        'eta_write': False,
        'c_write': False,
        'bs_write': False,
        'hz_write': False,
        'air_write': False,
        'cc_write': False,
        'primordial_write': False,
        'f_write': False,
        't_spectrum_write': False,
        'vm_write': False,
        'stress_write': False,
        'g_write': False,
        'tra_write': False,
        'metal_write': False,
        'vd_write': False,
        'edot_write': False,
        'age_write': False,
        'nmelt_write': False,
        'rho_write': False,
        'grainsize_write': False,
        'hpe_write': False,
        'water_write': False,
        'K_Ar_write': False,
        'oxides_write': False,
        'ph_write': False,
        'residue_write': False,
        'nrho_write': False,
        'w_write': False,
        'delete_old_trafiles': True,
        'delete_old_nonhdf_files': False,
        'torpol_write': False,
        'svel_write': False,
        'pvel_write': False,
        'bvel_write': False,
        'topo_write': False,
        'lyap_write': False,
        'divvor_write': False,
        'crust_write': False,
        'heatflux_write': False,
        'heatflux_spectrum_write': False,
        'lmax': 16,
        'stress_axis_write': False,
        'trapercell_write': False,
        'write_32bit_precision': False,
        'MORBprof_write': False,
        'save_file_framestep': 10,
        'overwrite_old_files': False,
        'restart_from_meshed_fields_only': False,
        'hdf5_output_folder': '+hdf5',
        'hdf5_input_folder': '+hdf5',
        'MaxFileSizeAllowed': 100000000,
        'MaxFileSizeAllowedTra': 100000000,
    },

    'compin': {
        'imode_comp': 'layered',
        'zcomp': 0.5,
        'amp_comp': 0.0,
        'ra_comp': [0.0],
        'lenardic_filter': False,
        'outC_val': -1.0,
        'dzcomp': 0.1,
    },

    'melt': {
        'imode_melt': 'whatever',
        'ra_melt': 0.0,
        'mr_melt': 1.e6,
        'Tsol0': 0.5,
        'dTsol_dz': 1.0,
        'latent_heat': 1.0,
        'amp_melt': 0.02,
        'eruption': False,
        'erupt_fraction': 1.0,
        'intrusion': False,
        'intrude_fraction': 1.0,
        'tzintrusion': False,
        'tzintrude_fraction': 1.0,
        'outF_val': -1.0,
        'erupt_mode': 'all',
        'solidus_function': 'simple',
        'd_intrude': 0.0,
        'intrude_position': 'surface',
        'harzburgite_melts': False,
        'metal_melts': True,
        'suppress_asth': False,
        'c_dependent_solidus': False,
        'd_erupt_Earth': 1.0,
        'erupt_threshold': 0.0,
        'deltaTsol_depletion_dimensional': 60.,
        'deltaTsol_water_dimensional': 0.,
        'ddens_sol_liq_dimensional': 500.,
        'eta_liquid_dimensional': 10.,
        'k0_dimensional': 3.e-10,
        'p_dependent_solidus': False,
        'Cf_dependent_Tmelting': False,
        'melt_phase_systems': False,
        'melt_segregation': False,
        'erupt_T_lith': 1400.,
        'magma_effective_tcond': False,
        'magma_tcond_factor': 1.e5,
        'magma_topTBL_const': 1.e-7,
        'fractional_xtallisation_n_melting': False,
        'trackTTGsource': False,
    },

    'phase': {
        'nphase_systems': 1,
        'Ol_frac_ref': 0.6,
        'basalt_frac_ref': 0.2,
        'effective_Cp_alpha': True,
        'scale_PCdepths': False,
        'g0_dimensional_PCdepths': 9.81,
        'D0_dimensional_PCdepths': 2890.e3,
        'sys_ol': 1,
        'sys_pxgt': 2,
        'sys_melt_ol': 3,
        'sys_melt_pxgt': 4,
        'sys_prim': 5,
        'sys_ccrust': 6,
        'sys_metal': 7,
    },

    'continents': {
        'ncont': 1,
        'cont_radius': [0.1],
        'cont_startpos': [0.5],
        'cont_offsetplot': True,
        'rafts_move': False,
        'cont_collideaction': 'ignore',
        'cont_Biot': 1.e6,
        'cont_thickness': 100.e3,
    },

    'tracersin': {
        'ntracers': 50,
        'ntracers_per_cell': -1,
        'imode_tra': 'even+ran',
        'B_basalt': 0.0,
        'B_ccrust': 0.0,
        'B_metal': 0.0,
        'd_crust': 0.1,
        'w_crust': 0.0,
        'Ctr_truncate': False,
        'k_fe': 0.85,
        'Fe_eut': 0.8,
        'Fe_cont': 0.1,
        'Ctr_lininterp': True,
        'Ttr_lininterp': True,
        'H2Otr_lininterp': True,
        'trastrain_diagnostics': False,
        'tracers_everywhere': True,
        'tracers_strain': False,
        'tracers_timemelt': False,
        'tracers_recstartpos': False,
        'tracers_advord': 2,
        'tracers_interp2ord': True,
        'tracers_weakcrust': False,
        'weakcrust_maxdepth': 0.3,
        'tracers_temperature': False,
        'oxides_compositions': False,
        'basalt_harzburgite': False,
        'continental_crust': False,
        'tracers_weakfault': False,
        'traceelement_hpe': False,
        'traceelement_water': False,
        'traceelement_K_Ar': False,
        'track_GrainSize': False,
        'metal': False,
        'ddens_SiO2': 0.0,
        'ddens_MgO': 0.0,
        'ddens_FeO': 0.0,
        'ddens_XO': 0.0,
        'dTmelting_dFeO': 0.0,
        'primordial_layer': True,
        'd_primordial': 0.1,
        'B_primordial': 0.0,
        'imode_primordial': 'layer',
        'imode_metal': 'top_layer',
        'd_metal': 0.0,
        'tracers_horizontaladvection': False,
        'tracers_noadvection': False,
        'Dpartition_FeO': 1.0,
        'Dpartition_hpe': 1.0,
        'Dpartition_water': 1.0,
        'Dpartition_K': 1.0,
        'Dpartition_Ar': 1.0,
        'outgas_fraction_water': 1.0,
        'outgas_fraction_Ar': 1.0,
        'ingas_water': False,
        'ingas_depth': 10.e3,
        'IntrudedBasaltHydrationFraction': 0.0,
        'overturn': False,
        'mode_init_overturn': 'linear',
        'Zdiff_h2o': 0.1,
        'diff_h2o': 0.5,
        'water_at_top': True,
        'topH2O_val': 0.0,
        'cH2O_mantle': 1.0,
        'diffusion_water': False,
    },

    'plot': {
        'plot_file_stem': 'image',
        'npix': 200000,
        'dots': True,
        'auto_t_scale': True,
        'z_xyslice': 0.5,
        't_superadiabatic': False,
        't_plot': True,
        'v_plot': False,
        'p_plot': False,
        'eta_plot': False,
        'rho_plot': False,
        'c_plot': False,
        'f_plot': False,
        'pd_plot': False,
        'dT_plot': False,
        'vm_plot': False,
        'stress_plot': False,
        'edot_plot': False,
        'g_plot': False,
        'tra_plot': False,
        'vd_plot': False,
        'air_plot': False,
        'oxides_plot': False,
        'hpe_plot': False,
        'water_plot': False,
        'K_Ar_plot': False,
        'primordial_plot': False,
        'rhs_plot': False,
        'bs_plot': False,
        'hz_plot': False,
        'cc_plot': False,
        'tcond_plot': False,
        'Cp_plot': False,
        'expan_plot': False,
        'grainsize_plot': False,
        'age_plot': False,
        'nmelt_plot': False,
        'strain_plot': False,
        'ph_plot': False,
        'lyap_plot': False,
        'residue_plot': False,
        'trapercell_plot': False,
        'imgtype': 'png',
    },

    'postprocessing': {
    },

    'atm_stuff': {
        'atm_model': 'simple',
    },

}


def _write_default():
    """create default par file"""
    if not PAR_DFLT_FILE.is_file():
        f90nml.write(PAR_DEFAULT, str(PAR_DFLT_FILE))


def _read_default():
    """read default par file"""
    _write_default()
    par_conf = f90nml.read(str(PAR_DFLT_FILE))
    for section in par_conf:
        PAR_DEFAULT[section].update(par_conf[section])
    return PAR_DEFAULT


def readpar(path):
    """read StagYY par file"""
    par_file = path / 'par'
    par_dflt = _read_default()
    if par_file.is_file():
        par_nml = f90nml.read(str(par_file))
        for section in par_nml:
            par_dflt[section].update(par_nml[section])
    else:
        print('No par file found! Check path.')
        sys.exit()
    par_nml = par_dflt
    return par_nml



"""plot fields"""

from inspect import getdoc
import numpy as np
from . import constants, misc
from .stagyydata import StagyyData


def valid_field_var(var):
    """Whether a field var is defined"""
    return var in constants.FIELD_VARS or var in constants.FIELD_VARS_EXTRA


def get_meshes_fld(step, var):
    """Return 2D meshes and field"""
    fld = step.fields[var]
    if step.geom.twod_xz:
        xmesh, ymesh = step.geom.x_mesh[:, 0, :], step.geom.z_mesh[:, 0, :]
        fld = fld[:, 0, :, 0]
    elif step.geom.cartesian and step.geom.twod_yz:
        xmesh, ymesh = step.geom.y_mesh[0, :, :], step.geom.z_mesh[0, :, :]
        fld = fld[0, :, :, 0]
    else:  # spherical yz
        xmesh, ymesh = step.geom.x_mesh[0, :, :], step.geom.y_mesh[0, :, :]
        fld = fld[0, :, :, 0]
    return xmesh, ymesh, fld


def get_meshes_vec(step, var):
    """Return 2D meshes and vec field components"""
    if step.geom.twod_xz:
        xmesh, ymesh = step.geom.x_mesh[:, 0, :], step.geom.z_mesh[:, 0, :]
        vec1 = step.fields[var + '1'][:, 0, :, 0]
        vec2 = step.fields[var + '3'][:, 0, :, 0]
    elif step.geom.cartesian and step.geom.twod_yz:
        xmesh, ymesh = step.geom.y_mesh[0, :, :], step.geom.z_mesh[0, :, :]
        vec1 = step.fields[var + '2'][0, :, :, 0]
        vec2 = step.fields[var + '3'][0, :, :, 0]
    else:  # spherical yz
        xmesh, ymesh = step.geom.x_mesh[0, :, :], step.geom.y_mesh[0, :, :]
        pmesh = step.geom.p_mesh[0, :, :]
        vec_phi = step.fields[var + '2'][0, :, :, 0]
        vec_r = step.fields[var + '3'][0, :, :, 0]
        vec1 = vec_r * np.cos(pmesh) - vec_phi * np.sin(pmesh)
        vec2 = vec_phi * np.cos(pmesh) + vec_r * np.sin(pmesh)
    return xmesh, ymesh, vec1, vec2


def set_of_vars(arg_plot):
    """List of vars

    Each var is a tuple, first component is a scalar field, second component is
    either:
    - a scalar field, isocontours are added to the plot
    - a vector field (e.g. 'v' for the (v1,v2,v3) vector), arrows are added to
      the plot
    """
    sovs = set(tuple((var + '+').split('+')[:2])
               for var in arg_plot.split(','))
    sovs.discard(('', ''))
    return sovs


def plot_scalar(args, step, var, scaling=None, **extra):
    """Plot scalar field"""
    plt = args.plt

    if var in constants.FIELD_VARS:
        meta = constants.FIELD_VARS[var]
    else:
        meta = constants.FIELD_VARS_EXTRA[var]
        meta = constants.Varf(getdoc(meta.description),
                              meta.shortname, meta.popts)
    if step.geom.threed:
        raise ValueError('plot_scalar only implemented for 2D fields')

    xmesh, ymesh, fld = get_meshes_fld(step, var)

    if scaling is not None:
        fld = np.copy(fld) * scaling

    fig, axis = plt.subplots(ncols=1)
    extra_opts = {'cmap': 'jet'}
    extra_opts.update(meta.popts)
    extra_opts.update({} if var != 'eta'
                      else {'norm': args.mpl.colors.LogNorm()})
    extra_opts.update(extra)
    surf = axis.pcolormesh(xmesh, ymesh, fld, rasterized=not args.pdf,
                           shading='gouraud', **extra_opts)

    cbar = plt.colorbar(surf, shrink=args.shrinkcb)
    cbar.set_label(r'${}$'.format(meta.shortname))
    plt.axis('equal')
    plt.axis('off')
    return fig, axis, surf, cbar


def plot_iso(axis, step, var):
    """Plot isocontours"""
    xmesh, ymesh, fld = get_meshes_fld(step, var)
    axis.contour(xmesh, ymesh, fld, linewidths=1)


def plot_vec(axis, step, var):
    """Plot vector field"""
    xmesh, ymesh, vec1, vec2 = get_meshes_vec(step, var)
    dip = step.geom.nztot // 10
    axis.quiver(xmesh[::dip, ::dip], ymesh[::dip, ::dip],
                vec1[::dip, ::dip], vec2[::dip, ::dip],
                linewidths=1)


def field_cmd(args):
    """extract and plot field data"""
    sdat = StagyyData(args.path)
    sovs = set_of_vars(args.plot)
    for step in misc.steps_gen(sdat, args):
        for var in sovs:
            if step.fields[var[0]] is None:
                print("'{}' field on snap {} not found".format(var,
                                                               step.isnap))
                continue
            fig, axis, _, _ = plot_scalar(args, step, var[0])
            if valid_field_var(var[1]):
                plot_iso(axis, step, var[1])
            elif var[1]:
                plot_vec(axis, step, var[1])
            oname = '{}_{}'.format(*var) if var[1] else var[0]
            fig.savefig(
                misc.out_name(args, oname).format(step.isnap) + '.pdf',
                format='PDF', bbox_inches='tight')
            args.plt.close(fig)



"""miscellaneous definitions"""

import importlib
import os
import shutil
import sys
import tempfile

INT_FMT = '{:05d}'


def stop(*msgs):
    """print error message and exit"""
    print('ERROR:', *msgs, file=sys.stderr)
    sys.exit()


def out_name(args, par_type):
    """return out file name format for any time step"""
    return args.outname + '_' + par_type + INT_FMT


def fmttime(tin):
    """Time formatting for labels"""
    aaa, bbb = '{:.2e}'.format(tin).split('e')
    bbb = int(bbb)
    return r'$t={} \times 10^{{{}}}$'.format(aaa, bbb)


def set_arg(args, arg, val):
    """set a cmd line with arg string name"""
    vars(args)[arg] = val


def get_arg(args, arg):
    """set a cmd line with arg string name"""
    return vars(args)[arg]


def list_of_vars(arg_plot):
    """Compute list of variables per plot

    Three nested lists:
    - variables on the same subplots;
    - subplots on the same figure;
    - figures.
    """
    lovs = [[[var for var in svars.split(',') if var]
             for svars in pvars.split('.') if svars]
            for pvars in arg_plot.split('_') if pvars]
    return [lov for lov in lovs if lov]


def set_of_vars(lovs):
    """Build set of variables from list"""
    return set(var for pvars in lovs for svars in pvars for var in svars)


def steps_gen(sdat, args):
    """Return generator over relevant snapshots or timesteps"""
    if args.snapshots is not None:
        return sdat.snaps[args.snapshots]
    else:
        return sdat.steps[args.timesteps]


def get_rbounds(step):
    """Radii of boundaries"""
    if step.geom is not None:
        rcmb = step.geom.rcmb
    else:
        rcmb = step.sdat.par['geometry']['r_cmb']
    rcmb = max(rcmb, 0)
    return rcmb, rcmb + 1


def plot_backend(args):
    """import matplotlib and seaborn"""
    mpl = importlib.import_module('matplotlib')
    if args.matplotback:
        mpl.use(args.matplotback)
    plt = importlib.import_module('matplotlib.pyplot')
    if args.useseaborn:
        sns = importlib.import_module('seaborn')
        args.sns = sns
    if args.xkcd:
        plt.xkcd()
    args.mpl = mpl
    args.plt = plt


class InchoateFiles:

    """Context manager handling files whose names are not known yet"""

    def __init__(self, nfiles=1, tmp_prefix=None):
        """Initialize context object

        nfiles: number of files
        tmp_prefix: prefix name of temporary files
        """
        self._fnames = ['inchoate{}'.format(i) for i in range(nfiles)]
        self._tmpprefix = tmp_prefix
        self._fids = []

    @property
    def fids(self):
        """List of files id"""
        return self._fids

    @property
    def fnames(self):
        """List of filenames"""
        return self._fnames

    @fnames.setter
    def fnames(self, names):
        """Ensure constant size of fnames"""
        names = list(names[:len(self._fnames)])
        self._fnames = names + self._fnames[len(names):]

    def __getitem__(self, idx):
        return self._fids[idx]

    def __enter__(self):
        """Create temporary files"""
        for ifile in range(len(self.fnames)):
            self._fids.append(
                tempfile.NamedTemporaryFile(
                    mode='w', prefix=self._tmpprefix, delete=False))
        return self

    def __exit__(self, *exc_info):
        """Give temporary files their final names"""
        for tmp in self._fids:
            tmp.close()
        if exc_info[0] is None:
            for fname, tmp in zip(self.fnames, self._fids):
                shutil.copyfile(tmp.name, fname)
        for tmp in self._fids:
            os.remove(tmp.name)



"""Plots radial profiles coming out of stagyy.

Author: Stephane Labrosse with inputs from Martina Ulvrova and Adrien Morison
Date: 2015/09/11
"""
from inspect import getdoc
from . import constants, misc
from .stagyydata import StagyyData


def _plot_rprof_list(lovs, rprofs, metas, args, stepstr, rads=None):
    """Plot requested profiles"""
    if rads is None:
        rads = {}
    for vfig in lovs:
        fig, axes = args.plt.subplots(ncols=len(vfig), sharey=True)
        axes = [axes] if len(vfig) == 1 else axes
        fname = ''
        for iplt, vplt in enumerate(vfig):
            xlabel = None
            for rvar in vplt:
                fname += rvar + '_'
                rad = rads[rvar] if rvar in rads else rprofs['r']
                axes[iplt].plot(rprofs[rvar], rad,
                                label=metas[rvar].description)
                if xlabel is None:
                    xlabel = metas[rvar].shortname
                elif xlabel != metas[rvar].shortname:
                    xlabel = ''
            if xlabel:
                axes[iplt].set_xlabel(r'${}$'.format(xlabel))
            if vplt[0][:3] == 'eta':  # list of log variables
                axes[iplt].set_xscale('log')
            axes[iplt].legend()
        axes[0].set_ylabel(r'$r$')
        fig.savefig('{}{}.pdf'.format(fname, stepstr),
                    format='PDF', bbox_inches='tight')
        args.plt.close(fig)


def get_rprof(step, var):
    """Return read or computed rprof along with metadata"""
    if var in step.rprof.columns:
        rprof = step.rprof[var]
        rad = None
        if var in constants.RPROF_VARS:
            meta = constants.RPROF_VARS[var]
        else:
            meta = constants.Varr(None, None)
    elif var in constants.RPROF_VARS_EXTRA:
        meta = constants.RPROF_VARS_EXTRA[var]
        rprof, rad = meta.description(step)
        meta = constants.Varr(getdoc(meta.description), meta.shortname)
    else:
        raise ValueError('Unknown rprof variable {}.'.format(var))

    return rprof, rad, meta


def plot_grid(step, args):
    """Plot cell position and thickness"""
    rad = step.rprof['r']
    drad, _, _ = get_rprof(step, 'dr')
    fig, (ax1, ax2) = args.plt.subplots(2, sharex=True)
    ax1.plot(rad, '-ko')
    ax1.set_ylabel('$r$')
    ax2.plot(drad, '-ko')
    ax2.set_ylabel('$dr$')
    ax2.set_xlim([-0.5, len(rad) - 0.5])
    ax2.set_xlabel('Cell number')
    fig.savefig('grid_{}.pdf'.format(step.istep))
    args.plt.close(fig)


def plot_average(sdat, lovs, args):
    """Plot time averaged profiles"""
    sovs = misc.set_of_vars(lovs)
    istart = None
    # assume constant z spacing for the moment
    ilast = sdat.rprof.index.levels[0][-1]
    rlast = sdat.rprof.loc[ilast]
    rprof_averaged = rlast.loc[:, sovs] * 0
    nprofs = 0
    rads = {}
    metas = {}
    for step in misc.steps_gen(sdat, args):
        if step.rprof is None:
            continue
        if istart is None:
            istart = step.istep
        ilast = step.istep
        nprofs += 1
        for rvar in sovs:
            rprof, rad, meta = get_rprof(step, rvar)
            rprof_averaged[rvar] += rprof
            metas[rvar] = meta
            if rad is not None:
                rads[rvar] = rad

    rprof_averaged /= nprofs
    rprof_averaged['r'] = rlast.loc[:, 'r'] + \
        misc.get_rbounds(sdat.steps[ilast])[0]

    stepstr = '{}_{}'.format(istart, ilast)

    _plot_rprof_list(lovs, rprof_averaged, metas, args, stepstr, rads)


def plot_every_step(sdat, lovs, args):
    """One plot per time step"""
    sovs = misc.set_of_vars(lovs)

    for step in misc.steps_gen(sdat, args):
        if step.rprof is None:
            continue
        rprofs = {}
        rads = {}
        metas = {}
        for rvar in sovs:
            rprof, rad, meta = get_rprof(step, rvar)
            rprofs[rvar] = rprof
            metas[rvar] = meta
            if rad is not None:
                rads[rvar] = rad
        rprofs['r'] = step.rprof.loc[:, 'r'] + misc.get_rbounds(step)[0]
        stepstr = str(step.istep)

        _plot_rprof_list(lovs, rprofs, metas, args, stepstr, rads)


def rprof_cmd(args):
    """Plot radial profiles"""
    sdat = StagyyData(args.path)
    if sdat.rprof is None:
        return

    if args.grid:
        for step in misc.steps_gen(sdat, args):
            plot_grid(step, args)

    lovs = misc.list_of_vars(args.plot)
    if not lovs:
        return

    if args.average:
        plot_average(sdat, lovs, args)
    else:
        plot_every_step(sdat, lovs, args)



"""Handle configuration of StagPy

Create the cmd line argument parser
and deal with the config file
"""

from collections import OrderedDict, namedtuple
from inspect import getdoc
from subprocess import call
import argcomplete
import argparse
import configparser
import shlex
from . import commands
from .constants import CONFIG_DIR

CONFIG_FILE = CONFIG_DIR / 'config'

Conf = namedtuple('ConfigEntry',
                  ['default', 'cmd_arg', 'shortname', 'kwargs',
                   'conf_arg', 'help_string'])

CORE = OrderedDict((
    ('path', Conf('./', True, 'p', {},
                  True, 'StagYY run directory')),
    ('outname', Conf('stagpy', True, 'n', {},
                     True, 'StagPy generic output file name')),
    ('timesteps', Conf(None, True, 't',
                       {'nargs': '?', 'const': ':', 'type': str},
                       False, 'timesteps slice')),
    ('snapshots', Conf(None, True, 's',
                       {'nargs': '?', 'const': ':', 'type': str},
                       False, 'snapshots slice')),
    ('xkcd', Conf(False, True, None, {},
                  True, 'use the xkcd style')),
    ('pdf', Conf(False, True, None, {},
                 True, 'produce non-rasterized pdf (slow!)')),
    ('fontsize', Conf(16, False, None, {},
                      True, 'font size')),
    ('linewidth', Conf(2, False, None, {},
                       True, 'line width')),
    ('matplotback', Conf('agg', False, None, {},
                         True, 'graphical backend')),
    ('useseaborn', Conf(True, False, None, {},
                        True, 'use or not seaborn')),
))

SCALING = OrderedDict((
    ('yearins', Conf(3.154e7, False, None, {},
                     True, 'Year in seconds')),
    ('ttransit', Conf(1.78e15, False, None, {},
                      True, 'Transit time in My')),
    ('kappa', Conf(1.0e-6, False, None, {},
                   True, 'Earth mantle thermal diffusivity m2/s')),
    ('mantle', Conf(2890.0e3, False, None, {},
                    True, 'Thickness of Earth mantle m')),
    ('viscosity_ref', Conf(5.86e22, False, None, {},
                           True, 'Reference viscosity Pa s')),
))

PLOTTING = OrderedDict((
    ('topomin', Conf(-40, False, None, {},
                     True, 'Min range for topography plots')),
    ('topomax', Conf(100, False, None, {},
                     True, 'Max range for topography plots')),
    ('agemin', Conf(-50, False, None, {},
                    True, 'Min range for age plots')),
    ('agemax', Conf(500, False, None, {},
                    True, 'Max range for age plots')),
    ('velocitymin', Conf(-5000, False, None, {},
                         True, 'Min range for velocity plots')),
    ('velocitymax', Conf(5000, False, None, {},
                         True, 'Max range for velocity plots')),
    ('dvelocitymin', Conf(-250000, False, None, {},
                          True, 'Min range for velocity derivative plots')),
    ('dvelocitymax', Conf(150000, False, None, {},
                          True, 'Max range for velocity derivative plots')),
    ('stressmin', Conf(0, False, None, {},
                       True, 'Min range for stress plots')),
    ('stressmax', Conf(800, False, None, {},
                       True, 'Max range for stress plots')),
    ('lstressmax', Conf(50, False, None, {},
                        True, 'Max range for lithospheric stress plots')),
))

FIELD = OrderedDict((
    ('plot',
        Conf('T+stream', True, 'o',
             {'nargs': '?', 'const': '', 'type': str},
             True, ('specify which variables to plot, '
                    'run stagpy var for a list of variables'))),
    ('shrinkcb',
        Conf(0.5, False, None, {},
             True, 'color bar shrink factor')),
))

RPROF = OrderedDict((
    ('plot',
        Conf('Tmean', True, 'o',
             {'nargs': '?', 'const': ''},
             True, 'specify which variables to plot')),
    ('average',
        Conf(False, True, 'a', {},
             True, 'Plot temporal average')),
    ('grid',
        Conf(False, True, 'g', {},
             True, 'Plot grid')),
))

TIME = OrderedDict((
    ('plot',
        Conf('Nutop,ebalance,Nubot.Tmean', True, 'o',
             {'nargs': '?', 'const': ''},
             True, 'specify which variables to plot')),
    ('compstat',
        Conf(False, True, None, {},
             True, 'compute steady state statistics')),
    ('tstart',
        Conf(0., True, None, {},
             False, 'specify beginning for the time series')),
    ('tend',
        Conf(None, True, None, {'type': float},
             False, 'specify end time for the time series')),
))

PLATES = OrderedDict((
    ('plot',
        Conf(None, True, 'o',
             {'nargs': '?', 'const': '', 'type': str},
             False, ('specify which variable to plot, '
                     'run stagpy var for a list of variables'))),
    ('plot_composition',
        Conf(True, False, None, {},
             True, 'composition scalar field')),
    ('plot_viscosity',
        Conf(True, False, None, {},
             True, 'viscosity scalar field')),
    ('plot_topography',
        Conf(True, False, None, {},
             True, 'topography scalar field')),
    ('plot_age',
        Conf(False, False, None, {},
             True, 'age scalar field')),
    ('plot_stress',
        Conf(False, False, None, {},
             True, 'second invariant of stress scalar field')),
    ('plot_deviatoric_stress',
        Conf(False, False, None, {},
             True, 'principal deviatoric stress')),
    ('plot_strainrate',
        Conf(False, False, None, {},
             True, 'strain rate scalar field')),
    ('vzcheck',
        Conf(False, True, None, {},
             True, 'activate Colin\'s version with vz checking')),
    ('timeprofile',
        Conf(False, True, None, {},
             True, 'plots nb of plates in function of time')),
    ('shrinkcb',
        Conf(0.5, False, None, {},
             True, 'color bar shrink factor')),
    ('zoom',
        Conf(None, True, None, {'type': float},
             False, 'Zoom around surface')),
))

INFO = OrderedDict((
))

VAR = OrderedDict((
))

VERSION = OrderedDict((
))

CONFIG = OrderedDict((
    ('create',
        Conf(None, True, None, {'action': 'store_true'},
             False, 'create new config file')),
    ('update',
        Conf(None, True, None, {'action': 'store_true'},
             False, 'add missing entries to existing config file')),
    ('edit',
        Conf(None, True, None, {'action': 'store_true'},
             False, 'open config file in a text editor')),
    ('editor',
        Conf('vim', False, None, {},
             True, 'text editor')),
))


def config_cmd(args):
    """Configuration handling"""
    if not (args.create or args.update or args.edit):
        args.update = True
    if args.create or args.update:
        create_config()
    if args.edit:
        call(shlex.split('{} {}'.format(args.editor, CONFIG_FILE)))

Sub = namedtuple('Sub', ['conf_dict', 'use_core', 'func'])
SUB_CMDS = OrderedDict((
    ('field', Sub(FIELD, True, commands.field_cmd)),
    ('rprof', Sub(RPROF, True, commands.rprof_cmd)),
    ('time', Sub(TIME, True, commands.time_cmd)),
    ('plates', Sub(PLATES, True, commands.plates_cmd)),
    ('info', Sub(INFO, True, commands.info_cmd)),
    ('var', Sub(VAR, False, commands.var_cmd)),
    ('version', Sub(VERSION, False, commands.version_cmd)),
    ('config', Sub(CONFIG, False, config_cmd)),
))

DummySub = namedtuple('DummySub', ['conf_dict'])
DUMMY_CMDS = OrderedDict((
    ('core', DummySub(CORE)),
))
DUMMY_CMDS.update(SUB_CMDS)


def _set_conf_default(conf_dict, opt, dflt):
    """set default value of option in conf_dict"""
    conf_dict[opt] = conf_dict[opt]._replace(default=dflt)


def _read_section(config_parser, sub_cmd, meta):
    """read section of config parser

    read section corresponding to the sub command sub_cmd
    and set meta.conf_dict default values to the read values
    """
    config_content = []
    missing_opts = []
    for opt, meta_opt in meta.conf_dict.items():
        if not config_parser.has_option(sub_cmd, opt):
            if meta_opt.conf_arg:
                missing_opts.append(opt)
            continue
        if isinstance(meta_opt.default, bool):
            dflt = config_parser.getboolean(sub_cmd, opt)
        elif isinstance(meta_opt.default, float):
            dflt = config_parser.getfloat(sub_cmd, opt)
        elif isinstance(meta_opt.default, int):
            dflt = config_parser.getint(sub_cmd, opt)
        else:
            dflt = config_parser.get(sub_cmd, opt)
        config_content.append((meta.conf_dict, opt, dflt))
    return config_content, missing_opts


def _report_missing_config(config_out):
    """Print list of missing entries in config file"""
    _, missing_opts, missing_sections = config_out
    need_update = False
    for sub_cmd, missing in missing_opts.items():
        if missing:
            print('WARNING! Missing options in {} section of config file:'.
                  format(sub_cmd), *missing, sep='\n', end='\n\n')
            need_update = True
    if missing_sections:
        print('WARNING! Missing sections in config file:', *missing_sections,
              sep='\n', end='\n\n')
        need_update = True
    if need_update:
        print('Run stagpy config --update to update config file', end='\n\n')


def create_config():
    """Create config file"""
    if not CONFIG_DIR.exists():
        CONFIG_DIR.mkdir(parents=True)
    config_parser = configparser.ConfigParser()
    for sub_cmd, meta in DUMMY_CMDS.items():
        config_parser.add_section(sub_cmd)
        for opt, opt_meta in meta.conf_dict.items():
            if opt_meta.conf_arg:
                config_parser.set(sub_cmd, opt, str(opt_meta.default))
    with CONFIG_FILE.open('w') as out_stream:
        config_parser.write(out_stream)


def read_config():
    """Read config file and set conf_dict as needed"""
    if not CONFIG_FILE.is_file():
        return
    config_parser = configparser.ConfigParser()
    config_parser.read(str(CONFIG_FILE))
    config_content = []
    missing_sections = []
    missing_opts = {}
    for sub_cmd, meta in DUMMY_CMDS.items():
        if not config_parser.has_section(sub_cmd):
            missing_sections.append(sub_cmd)
            continue
        content, missing_opts[sub_cmd] = _read_section(config_parser, sub_cmd,
                                                       meta)
        config_content.extend(content)
    return config_content, missing_opts, missing_sections


class Toggle(argparse.Action):

    """argparse Action to store True/False to a +/-arg"""

    def __call__(self, parser, namespace, values, option_string=None):
        """set args attribute with True/False"""
        setattr(namespace, self.dest, bool('-+'.index(option_string[0])))


def add_args(parser, conf_dict):
    """Add arguments to a parser"""
    for arg, conf in conf_dict.items():
        if not conf.cmd_arg:
            continue
        if isinstance(conf.default, bool):
            conf.kwargs.update(action=Toggle, nargs=0)
            names = ['-{}'.format(arg), '+{}'.format(arg)]
            if conf.shortname is not None:
                names.append('-{}'.format(conf.shortname))
                names.append('+{}'.format(conf.shortname))
        else:
            if conf.default is not None:
                conf.kwargs.setdefault('type', type(conf.default))
            names = ['--{}'.format(arg)]
            if conf.shortname is not None:
                names.append('-{}'.format(conf.shortname))
        conf.kwargs.update(help=conf.help_string)
        parser.add_argument(*names, **conf.kwargs)
    parser.set_defaults(**{a: c.default for a, c in conf_dict.items()})
    return parser


def _build_parser():
    """Return complete parser"""
    main_parser = argparse.ArgumentParser(
        description='read and process StagYY binary data')
    main_parser.set_defaults(func=lambda _: print('stagpy -h for usage'))
    subparsers = main_parser.add_subparsers()

    core_parser = argparse.ArgumentParser(add_help=False, prefix_chars='-+')
    core_parser = add_args(core_parser, CORE)
    core_parser = add_args(core_parser, SCALING)
    core_parser = add_args(core_parser, PLOTTING)

    for sub_cmd, meta in SUB_CMDS.items():
        kwargs = {'prefix_chars': '+-', 'help': getdoc(meta.func)}
        if meta.use_core:
            kwargs.update(parents=[core_parser])
        dummy_parser = subparsers.add_parser(sub_cmd, **kwargs)
        dummy_parser = add_args(dummy_parser, meta.conf_dict)
        dummy_parser.set_defaults(func=meta.func)

    return main_parser


def _steps_to_slices(args):
    """parse timesteps and snapshots arguments and return slices"""
    if args.timesteps is None and args.snapshots is None:
        # default to the last snap
        args.snapshots = slice(-1, None, None)
        return None
    elif args.snapshots is not None:
        # snapshots take precedence over timesteps
        # if both are defined
        args.timesteps = None
        steps = args.snapshots
    else:
        steps = args.timesteps
    steps = steps.split(':')
    steps[0] = int(steps[0]) if steps[0] else None
    if len(steps) == 1:
        steps.append(steps[0] + 1)
    steps[1] = int(steps[1]) if steps[1] else None
    if len(steps) != 3:
        steps = steps[0:2] + [1]
    steps[2] = int(steps[2]) if steps[2] else None
    steps = slice(*steps)
    if args.snapshots is not None:
        args.snapshots = steps
    else:
        args.timesteps = steps


def parse_args():
    """Parse cmd line arguments"""
    main_parser = _build_parser()
    argcomplete.autocomplete(main_parser)
    args = main_parser.parse_args()

    if args.func is not config_cmd:
        args.create = False
        args.edit = False
        args.update = False

    try:
        config_out = read_config()
    except configparser.Error:
        config_out = None

    if not args.create or args.edit:
        if config_out is None:
            print('Unable to read config file {}!'.format(CONFIG_FILE),
                  'Run stagpy config --create to obtain a new config file.',
                  '=' * 26, sep='\n')
        elif config_out and not args.update:
            _report_missing_config(config_out)
            for conf_dict, opt, dflt in config_out[0]:
                _set_conf_default(conf_dict, opt, dflt)
                main_parser = _build_parser()
                args = main_parser.parse_args()

    try:
        _steps_to_slices(args)
    except AttributeError:
        pass
    return args



"""Define high level structure StagyyData"""

import re
import pathlib
import numpy as np
from . import constants, parfile, stagyyparsers


UNDETERMINED = object()
# dummy object with a unique identifier,
# useful to mark stuff as yet undetermined,
# as opposed to either some value or None if
# non existent


class Error(Exception):

    """Base class for exceptions raised in this module"""

    pass


class NoSnapshotError(Error):

    """Raised when last snapshot is required but none exists"""

    pass


class _Geometry:

    """Geometry information"""

    _regexes = (re.compile(r'^n([xyztprb])tot$'),  # ntot
                re.compile(r'^([xyztpr])_coord$'),  # coord
                re.compile(r'^([xyz])_mesh$'),  # cartesian mesh
                re.compile(r'^([tpr])_mesh$'))  # curvilinear mesh

    def __init__(self, header, par):
        self._header = header
        self._par = par
        self._coords = None
        self._cart_meshes = None
        self._curv_meshes = None
        self._shape = {'sph': False, 'cyl': False, 'axi': False,
                       'ntot': list(header['nts']) + [header['ntb']]}
        self._init_shape()

        self._coords = [header['e1_coord'],
                        header['e2_coord'],
                        header['e3_coord']]

        # instead of adding horizontal rows, should construct two grids:
        # - center of cells coordinates (the current one);
        # - vertices coordinates on which vector fields are determined,
        #   which geometrically contains one more row.

        # add theta, phi / x, y row to have a continuous field
        if not self.twod_yz:
            self._coords[0] = np.append(
                self.t_coord,
                self.t_coord[-1] + self.t_coord[1] - self.t_coord[0])
        if not self.twod_xz:
            self._coords[1] = np.append(
                self.p_coord,
                self.p_coord[-1] + self.p_coord[1] - self.p_coord[0])

        if self.cartesian:
            self._cart_meshes = np.meshgrid(self.x_coord, self.y_coord,
                                            self.z_coord, indexing='ij')
            self._curv_meshes = (None, None, None)
        else:
            if self.twod_yz:
                self._coords[0] = np.array(np.pi / 2)
            elif self.twod_xz:
                self._coords[1] = np.array(0)
            t_mesh, p_mesh, r_mesh = np.meshgrid(
                self.x_coord, self.y_coord, self.z_coord + self.rcmb,
                indexing='ij')
            # compute cartesian coordinates
            # z along rotation axis at theta=0
            # x at th=90, phi=0
            # y at th=90, phi=90
            x_mesh = r_mesh * np.cos(p_mesh) * np.sin(t_mesh)
            y_mesh = r_mesh * np.sin(p_mesh) * np.sin(t_mesh)
            z_mesh = r_mesh * np.cos(t_mesh)
            self._cart_meshes = (x_mesh, y_mesh, z_mesh)
            self._curv_meshes = (t_mesh, p_mesh, r_mesh)

    def _init_shape(self):
        """Determine shape of geometry"""
        shape = self._par['geometry']['shape'].lower()
        aspect = self._header['aspect']
        if self.rcmb is not None and self.rcmb >= 0:
            # curvilinear
            self._shape['cyl'] = self.twod_xz and (shape == 'cylindrical' or
                                                   aspect[0] >= np.pi)
            self._shape['sph'] = not self._shape['cyl']
        elif self.rcmb is None:
            self._header['rcmb'] = self._par['geometry']['r_cmb']
            if self.rcmb >= 0:
                if self.twod_xz and shape == 'cylindrical':
                    self._shape['cyl'] = True
                elif shape == 'spherical':
                    self._shape['sph'] = True
        self._shape['axi'] = self.cartesian and self.twod_xz and \
            shape == 'axisymmetric'

    @property
    def cartesian(self):
        """Cartesian geometry"""
        return not self.curvilinear

    @property
    def curvilinear(self):
        """Spherical or cylindrical geometry"""
        return self.spherical or self.cylindrical

    @property
    def cylindrical(self):
        """Cylindrical geometry (2D spherical)"""
        return self._shape['cyl']

    @property
    def spherical(self):
        """Spherical geometry"""
        return self._shape['sph']

    @property
    def yinyang(self):
        """Yin-yang geometry (3D spherical)"""
        return self.spherical and self.nbtot == 2

    @property
    def twod_xz(self):
        """XZ plane only"""
        return self.nytot == 1

    @property
    def twod_yz(self):
        """YZ plane only"""
        return self.nxtot == 1

    @property
    def twod(self):
        """2D geometry"""
        return self.twod_xz or self.twod_yz

    @property
    def threed(self):
        """3D geometry"""
        return not self.twod

    def __getattr__(self, attr):
        # provide nDtot, D_coord, D_mesh and nbtot
        # with D = x, y, z or t, p, r
        for reg, dat in zip(self._regexes, (self._shape['ntot'],
                                            self._coords,
                                            self._cart_meshes,
                                            self._curv_meshes)):
            match = reg.match(attr)
            if match is not None:
                return dat['xtypzrb'.index(match.group(1)) // 2]
        return self._header[attr]


class _Fields(dict):

    """Wrap fields of a step"""

    def __init__(self, step):
        self.step = step
        self._header = UNDETERMINED
        self._geom = UNDETERMINED
        super().__init__()

    def __missing__(self, name):
        if name in constants.FIELD_VARS:
            filestem = ''
            for filestem, list_fvar in constants.FIELD_FILES.items():
                if name in list_fvar:
                    break
            fieldfile = self.step.sdat.filename(filestem, self.step.isnap)
        elif name in constants.FIELD_VARS_EXTRA:
            self[name] = constants.FIELD_VARS_EXTRA[name].\
                description(self.step)
            return self[name]
        else:
            raise ValueError("Unknown field variable: '{}'".format(name))
        parsed_data = stagyyparsers.fields(fieldfile)
        if parsed_data is None:
            return None
        header, fields = parsed_data
        self._header = header
        fld_names = constants.FIELD_FILES[filestem]
        for fld_name, fld in zip(fld_names, fields):
            if self._header['xyp'] == 0:
                if not self.geom.twod_yz:
                    newline = (fld[:1, :, :, :] + fld[-1:, :, :, :]) / 2
                    fld = np.concatenate((fld, newline), axis=0)
                if not self.geom.twod_xz:
                    newline = (fld[:, :1, :, :] + fld[:, -1:, :, :]) / 2
                    fld = np.concatenate((fld, newline), axis=1)
            self[fld_name] = fld
        return self[name]

    @property
    def geom(self):
        """Header info from bin file"""
        if self._header is UNDETERMINED:
            binfiles = self.step.sdat.binfiles_set(self.step.isnap)
            if binfiles:
                self._header = stagyyparsers.fields(binfiles.pop(),
                                                    only_header=True)
            else:
                self._header = None
        if self._geom is UNDETERMINED:
            if self._header is None:
                self._geom = None
            else:
                self._geom = _Geometry(self._header, self.step.sdat.par)
        return self._geom


class _Step:

    """Time step data structure"""

    def __init__(self, istep, sdat):
        self.istep = istep
        self.sdat = sdat
        self.fields = _Fields(self)
        self._isnap = UNDETERMINED

    @property
    def geom(self):
        """Geometry object"""
        return self.fields.geom

    @property
    def timeinfo(self):
        """Relevant time series data"""
        if self.istep in self.sdat.tseries.index:
            return self.sdat.tseries.loc[self.istep]
        else:
            return None

    @property
    def rprof(self):
        """Relevant radial profiles data"""
        if self.istep in self.sdat.rprof.index.levels[0]:
            return self.sdat.rprof.loc[self.istep]
        else:
            return None

    @property
    def isnap(self):
        """Fields snap corresponding to time step"""
        if self._isnap is UNDETERMINED:
            istep = None
            isnap = -1
            # could be more efficient if do 0 and -1 then bisection
            # (but loose intermediate <- would probably use too much
            # memory for what it's worth if search algo is efficient)
            while (istep is None or istep < self.istep) and isnap < 99999:
                isnap += 1
                istep = self.sdat.snaps[isnap].istep
                self.sdat.snaps.bind(isnap, istep)
                # all intermediate istep could have their ._isnap to None
            if istep != self.istep:
                self._isnap = None
        return self._isnap

    @isnap.setter
    def isnap(self, isnap):
        """Fields snap corresponding to time step"""
        try:
            self._isnap = int(isnap)
        except ValueError:
            pass


class _EmptyStep(_Step):

    """Dummy step object for nonexistent snaps"""

    def __init__(self):
        super().__init__(None, None)

    def __getattribute__(self, name):
        return None


class _Steps(dict):

    """Implement the .steps[istep] accessor"""

    def __init__(self, sdat):
        self.sdat = sdat
        self._last = UNDETERMINED
        super().__init__()

    def __setitem__(self, key, value):
        pass

    def __getitem__(self, key):
        try:
            # slice
            idxs = key.indices(self.last.istep + 1)
            return (super(self.__class__, self).__getitem__(k)
                    for k in range(*idxs))
        except AttributeError:
            return super().__getitem__(key)

    def __missing__(self, istep):
        if istep is None:  # if called for nonexistent snap
            return _EmptyStep()
        try:
            istep = int(istep)
        except ValueError:
            raise ValueError('Time step should be an integer value')
        if istep < 0:
            istep += self.last.istep + 1
            if istep < 0:
                raise ValueError('Time step should be positive')
        if not self.__contains__(istep):
            super().__setitem__(istep, _Step(istep, self.sdat))
        return super().__getitem__(istep)

    @property
    def last(self):
        """Last timestep available"""
        if self._last is UNDETERMINED:
            # not necessarily the last one...
            self._last = self.sdat.tseries.index[-1]
        return self[self._last]


class _Snaps(_Steps):

    """Implement the .snaps[isnap] accessor"""

    def __init__(self, sdat):
        self._isteps = {}
        super().__init__(sdat)

    def __getitem__(self, key):
        try:
            # slice
            idxs = key.indices(self.last.isnap + 1)
            return (self.__missing__(k) for k in range(*idxs))
        except AttributeError:
            return self.__missing__(key)

    def __missing__(self, isnap):
        if isnap < 0:
            isnap += self.last.isnap + 1
        istep = self._isteps.get(isnap, UNDETERMINED)
        if istep is UNDETERMINED:
            binfiles = self.sdat.binfiles_set(isnap)
            if binfiles:
                istep = stagyyparsers.fields(binfiles.pop(), only_istep=True)
            else:
                istep = None
            if istep is not None:
                self.bind(isnap, istep)
            else:
                self._isteps[isnap] = None
        return self.sdat.steps[istep]

    def bind(self, isnap, istep):
        """Make the isnap <-> istep link"""
        self._isteps[isnap] = istep
        self.sdat.steps[istep].isnap = isnap

    @property
    def last(self):
        """Last snapshot available"""
        if self._last is UNDETERMINED:
            self._last = -1
            out_stem = re.escape(pathlib.Path(
                self.sdat.par['ioin']['output_file_stem'] + '_').name[:-1])
            rgx = re.compile('^{}_([a-zA-Z]+)([0-9]{{5}})$'.format(out_stem))
            fstems = set(fstem for fstem in constants.FIELD_FILES)
            for fname in self.sdat.files:
                match = rgx.match(fname.name)
                if match is not None and match.group(1) in fstems:
                    self._last = max(int(match.group(2)), self._last)
            if self._last < 0:
                raise NoSnapshotError
        return self[self._last]


class StagyyData:

    """Offer a generic interface to StagYY output data"""

    def __init__(self, path):
        """Generic lazy StagYY output data accessors"""
        self.path = pathlib.Path(path)
        self.par = parfile.readpar(self.path)
        self.steps = _Steps(self)
        self.snaps = _Snaps(self)
        self._files = UNDETERMINED
        self._tseries = UNDETERMINED
        self._rprof = UNDETERMINED

    @property
    def tseries(self):
        """Time series data"""
        if self._tseries is UNDETERMINED:
            timefile = self.filename('time.dat')
            self._tseries = stagyyparsers.time_series(
                timefile, list(constants.TIME_VARS.keys()))
        return self._tseries

    @property
    def _rprof_and_times(self):
        if self._rprof is UNDETERMINED:
            rproffile = self.filename('rprof.dat')
            self._rprof = stagyyparsers.rprof(
                rproffile, list(constants.RPROF_VARS.keys()))
        return self._rprof

    @property
    def rprof(self):
        """Radial profiles data"""
        return self._rprof_and_times[0]

    @property
    def rtimes(self):
        """Radial profiles data"""
        return self._rprof_and_times[1]

    @property
    def files(self):
        """Set of output binary files"""
        if self._files is UNDETERMINED:
            out_stem = pathlib.Path(self.par['ioin']['output_file_stem'] + '_')
            out_dir = self.path / out_stem.parent
            self._files = set(out_dir.iterdir())
        return self._files

    def tseries_between(self, tstart=0., tend=None):
        """Time series data between requested times"""
        if self.tseries is None:
            return None

        ndat = self.tseries.shape[0]

        if tstart <= 0.:
            istart = 0
        else:
            igm = 0
            igp = ndat - 1
            while igp - igm > 1:
                istart = igm + (igp - igm) // 2
                if self.tseries.iloc[istart]['t'] >= tstart:
                    igp = istart
                else:
                    igm = istart
            istart = igp

        if tend is None:
            iend = None
        else:
            igm = 0
            igp = ndat - 1
            while igp - igm > 1:
                iend = igm + (igp - igm) // 2
                if self.tseries.iloc[iend]['t'] > tend:
                    igp = iend
                else:
                    igm = iend
            iend = igm + 1

        return self.tseries.iloc[istart:iend]

    def filename(self, fname, timestep=None, suffix=''):
        """return name of StagYY out file"""
        if timestep is not None:
            fname += '{:05d}'.format(timestep)
        fname = self.par['ioin']['output_file_stem'] + '_' + fname + suffix
        return self.path / fname

    def binfiles_set(self, isnap):
        """Set of existing binary files at a given snap"""
        possible_files = set(self.filename(fstem, isnap)
                             for fstem in constants.FIELD_FILES)
        return possible_files & self.files



"""Plots time series of temperature and heat fluxes outputs from stagyy.

Author: Stephane Labrosse with inputs from Martina Ulvrova and Adrien Morison
Date: 2015/11/27
"""
from inspect import getdoc
import numpy as np
from math import sqrt
from . import constants, misc
from .stagyydata import StagyyData


def _plot_time_list(lovs, tseries, metas, args, times=None):
    """Plot requested profiles"""
    if times is None:
        times = {}
    for vfig in lovs:
        fig, axes = args.plt.subplots(nrows=len(vfig), sharex=True,
                                      figsize=(30, 5 * len(vfig)))
        axes = [axes] if len(vfig) == 1 else axes
        fname = ''
        for iplt, vplt in enumerate(vfig):
            ylabel = None
            for tvar in vplt:
                fname += tvar + '_'
                time = times[tvar] if tvar in times else tseries['t']
                axes[iplt].plot(time, tseries[tvar],
                                label=metas[tvar].description,
                                linewidth=args.linewidth)
                lbl = metas[tvar].shortname
                if ylabel is None:
                    ylabel = lbl
                elif ylabel != lbl:
                    ylabel = ''
            if ylabel:
                axes[iplt].set_ylabel(r'${}$'.format(ylabel),
                                      fontsize=args.fontsize)
            if vplt[0][:3] == 'eta':  # list of log variables
                axes[iplt].set_yscale('log')
            axes[iplt].legend(fontsize=args.fontsize)
            axes[iplt].tick_params(labelsize=args.fontsize)
        axes[-1].set_xlabel(r'$t$', fontsize=args.fontsize)
        axes[-1].set_xlim((tseries['t'].iloc[0], tseries['t'].iloc[-1]))
        axes[-1].tick_params(labelsize=args.fontsize)
        fig.savefig('time_{}.pdf'.format(fname[:-1]),
                    format='PDF', bbox_inches='tight')
        args.plt.close(fig)


def get_time_series(sdat, var, tstart, tend):
    """Return read or computed time series along with metadata"""
    tseries = sdat.tseries_between(tstart, tend)
    if var in tseries.columns:
        series = tseries[var]
        time = None
        if var in constants.TIME_VARS:
            meta = constants.TIME_VARS[var]
        else:
            meta = constants.Varr(None, None)
    elif var in constants.TIME_VARS_EXTRA:
        meta = constants.TIME_VARS_EXTRA[var]
        series, time = meta.description(sdat, tstart, tend)
        meta = constants.Varr(getdoc(meta.description), meta.shortname)
    else:
        raise ValueError('Unknown time variable {}.'.format(var))

    return series, time, meta


def plot_time_series(sdat, lovs, args):
    """Plot requested time series"""
    sovs = misc.set_of_vars(lovs)
    tseries = {}
    times = {}
    metas = {}
    for tvar in sovs:
        series, time, meta = get_time_series(sdat, tvar,
                                             args.tstart, args.tend)
        tseries[tvar] = series
        metas[tvar] = meta
        if time is not None:
            times[tvar] = time
    tseries['t'] = sdat.tseries['t']

    _plot_time_list(lovs, tseries, metas, args, times)


def compstat(sdat, tstart=0., tend=None):
    """Compute statistics"""
    data = sdat.tseries_between(tstart, tend)
    time = data['t'].values

    moy = []
    rms = []
    delta_time = time[-1] - time[0]
    for col in data.columns[1:]:
        moy.append(np.trapz(data[col], x=time) / delta_time)
        rms.append(sqrt(np.trapz((data[col] - moy[-1])**2, x=time) /
                        delta_time))
    results = moy + rms
    with open('statistics.dat', 'w') as out_file:
        for item in results:
            out_file.write("%10.5e " % item)
        out_file.write("\n")


def time_cmd(args):
    """plot temporal series"""
    sdat = StagyyData(args.path)
    if sdat.tseries is None:
        return

    lovs = misc.list_of_vars(args.plot)
    if lovs:
        plot_time_series(sdat, lovs, args)

    if args.compstat:
        compstat(sdat, args.tstart, args.tend)



"""Various computations"""

import numpy as np
from scipy import integrate
from . import misc


def dt_dt(sdat, tstart=0., tend=None):
    """Derivative of temperature"""
    tseries = sdat.tseries_between(tstart, tend)
    time = tseries['t'].values
    temp = tseries['Tmean'].values
    dtdt = (temp[2:] - temp[:-2]) / (time[2:] - time[:-2])
    return dtdt, time[1:-1]


def ebalance(sdat, tstart=0., tend=None):
    """Energy balance"""
    tseries = sdat.tseries_between(tstart, tend)
    rbot, rtop = misc.get_rbounds(sdat.steps.last)
    if rbot != 0:  # spherical
        coefsurf = (rtop / rbot)**2
        volume = rbot * (1 - (rtop / rbot)**3) / 3
    else:
        coefsurf = 1.
        volume = 1.
    dtdt, time = dt_dt(sdat, tstart, tend)
    ftop = tseries['ftop'].values * coefsurf
    fbot = tseries['fbot'].values
    ebal = ftop[1:-1] - fbot[1:-1] - volume * dtdt
    return ebal, time


def r_edges(step):
    """Cell border"""
    rbot, rtop = misc.get_rbounds(step)
    centers = step.rprof.loc[:, 'r'].values + rbot
    # assume walls are mid-way between T-nodes
    # could be T-nodes at center between walls
    edges = (centers[:-1] + centers[1:]) / 2
    edges = np.insert(edges, 0, rbot)
    edges = np.append(edges, rtop)
    return edges, edges


def delta_r(step):
    """Cells thickness"""
    edges, _ = r_edges(step)
    return (edges[1:] - edges[:-1]), None


def _scale_prof(step, rprof, rad=None):
    """Scale profile to take sphericity into account"""
    rbot, rtop = misc.get_rbounds(step)
    if rbot == 0:  # not spherical
        return rprof
    if rad is None:
        rad = step.rprof['r'].values + rbot
    return rprof * (rad / rtop)**2


def diff_prof(step):
    """Diffusion"""
    rbot, rtop = misc.get_rbounds(step)
    rad = step.rprof['r'].values + rbot
    tprof = step.rprof['Tmean'].values
    diff = (tprof[:-1] - tprof[1:]) / (rad[1:] - rad[:-1])
    # assume tbot = 1
    diff = np.insert(diff, 0, (1 - tprof[0]) / (rad[0] - rbot))
    # assume ttop = 0
    diff = np.append(diff, tprof[-1] / (rtop - rad[-1]))
    # actually computed at r_edges...
    return diff, np.append(rad, rtop)


def diffs_prof(step):
    """Scaled diffusion"""
    diff, rad = diff_prof(step)
    return _scale_prof(step, diff, rad), rad


def advts_prof(step):
    """Scaled advection"""
    return _scale_prof(step, step.rprof['advtot']), None


def advds_prof(step):
    """Scaled downward advection"""
    return _scale_prof(step, step.rprof['advdesc']), None


def advas_prof(step):
    """Scaled upward advection"""
    return _scale_prof(step, step.rprof['advasc']), None


def energy_prof(step):
    """Energy flux"""
    diff, rad = diffs_prof(step)
    adv, _ = advts_prof(step)
    return (diff + np.append(adv, 0)), rad


def init_c_overturn(step):
    """Initial concentration"""
    rbot, rtop = misc.get_rbounds(step)
    xieut = step.sdat.par['tracersin']['fe_eut']
    k_fe = step.sdat.par['tracersin']['k_fe']
    xi0l = step.sdat.par['tracersin']['fe_cont']
    xi0s = k_fe * xi0l
    xired = xi0l / xieut
    rsup = (rtop**3 - xired**(1 / (1 - k_fe)) *
            (rtop**3 - rbot**3))**(1 / 3)

    def initprof(rpos):
        """Theoretical initial profile."""
        if rpos < rsup:
            return xi0s * ((rtop**3 - rbot**3) /
                           (rtop**3 - rpos**3))**(1 - k_fe)
        else:
            return xieut

    rad = np.linspace(rbot, rtop, 500)
    initprof = np.vectorize(initprof)
    return initprof(rad), rad


def c_overturned(step):
    """Theoretical overturned concentration"""
    rbot, rtop = misc.get_rbounds(step)
    cinit, rad = init_c_overturn(step)
    radf = (rtop**3 + rbot**3 - rad**3)**(1 / 3)
    return cinit, radf


def stream_function(step):
    """Stream function"""
    if step.geom.twod_yz:
        x_coord = step.geom.y_coord
        v_x = step.fields['v2'][0, :, :, 0]
        v_z = step.fields['v3'][0, :, :, 0]
        shape = (1, v_x.shape[0], v_x.shape[1], 1)
    elif step.geom.twod_xz and step.geom.cartesian:
        x_coord = step.geom.x_coord
        v_x = step.fields['v1'][:, 0, :, 0]
        v_z = step.fields['v3'][:, 0, :, 0]
        shape = (v_x.shape[0], 1, v_x.shape[1], 1)
    else:
        raise ValueError('Stream function only implemented in '
                         '2D cartesian and spherical annulus')
    psi = np.zeros_like(v_x)
    if step.geom.spherical:  # YZ annulus
        r_coord = step.geom.r_coord + step.geom.rcmb
        psi[0, :] = integrate.cumtrapz(r_coord * v_x[0, :],
                                       x=r_coord,
                                       initial=0)
        for i_z, r_pos in enumerate(r_coord):
            psi[:, i_z] = psi[0, i_z] - \
                integrate.cumtrapz(r_pos**2 * v_z[:, i_z],
                                   x=x_coord, initial=0)
    else:  # assume cartesian geometry
        psi[0, :] = integrate.cumtrapz(v_x[0, :],
                                       x=step.geom.z_coord,
                                       initial=0)
        for i_z in range(step.geom.nztot):
            psi[:, i_z] = psi[0, i_z] - integrate.cumtrapz(v_z[:, i_z],
                                                           x=x_coord,
                                                           initial=0)
    if step.geom.twod_xz:
        psi = - psi
    psi = np.reshape(psi, shape)
    return psi



"""define current version"""

from setuptools_scm import get_version
from pkg_resources import get_distribution, DistributionNotFound

try:
    __version__ = get_version(root='..', relative_to=__file__)
except LookupError:
    __version__ = get_distribution('stagpy').version
except (DistributionNotFound, ValueError):
    __version__ = 'unknown'



# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as rdm:
    README = rdm.read()

setup(
    name='stagpy',
    use_scm_version=True,

    description='Tool for StagYY output files processing',
    long_description=README,

    url='https://github.com/StagPython/StagPy',

    author='Martina Ulvrova, Adrien Morison, Stéphane Labrosse',
    author_email='adrien.morison@gmail.com',

    license='GPLv2',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        ],

    packages = ['stagpy'],
    entry_points = {
        'console_scripts': ['stagpy = stagpy.__main__:main']
        },
    setup_requires=['setuptools_scm'],
    install_requires = [
        'numpy',
        'scipy',
        'f90nml',
        'pandas',
        'matplotlib',
        'seaborn',
        'argcomplete',
        'setuptools_scm',
        ],
)
